"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file

* Copyright 2021 The Feast Authors
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
*     https://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class SqliteTable(google.protobuf.message.Message):
    """Represents a Sqlite table"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATH_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    path: builtins.str
    """Absolute path of the table"""
    name: builtins.str
    """Name of the table"""
    def __init__(
        self,
        *,
        path: builtins.str = ...,
        name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name", b"name", "path", b"path"]) -> None: ...

global___SqliteTable = SqliteTable



import enum


class AuthzedAction(enum.Enum):
    """
    Identify the type of action being secured by the permissions framework, according to the familiar CRUD and Feast terminology.
    """

    CREATE = "create"  # Create an instance
    DESCRIBE = "describe"  # Access the instance state
    UPDATE = "update"  # Update the instance state
    DELETE = "delete"  # Delete an instance
    READ_ONLINE = "read_online"  # Read the online store only
    READ_OFFLINE = "read_offline"  # Read the offline store only
    WRITE_ONLINE = "write_online"  # Write to the online store only
    WRITE_OFFLINE = "write_offline"  # Write to the offline store only


#  Alias for all available actions
ALL_ACTIONS = [a for a in AuthzedAction.__members__.values()]

#  Alias for all read actions
READ = [
    AuthzedAction.READ_OFFLINE,
    AuthzedAction.READ_ONLINE,
]
#  Alias for all write actions
WRITE = [
    AuthzedAction.WRITE_OFFLINE,
    AuthzedAction.WRITE_ONLINE,
]


#  Alias for CRUD actions
CRUD = [
    AuthzedAction.CREATE,
    AuthzedAction.DESCRIBE,
    AuthzedAction.UPDATE,
    AuthzedAction.DELETE,
]



import logging
from datetime import datetime
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple

import faiss
import numpy as np
from google.protobuf.timestamp_pb2 import Timestamp

from feast import Entity, FeatureView, RepoConfig
from feast.infra.key_encoding_utils import serialize_entity_key
from feast.infra.online_stores.online_store import OnlineStore
from feast.protos.feast.types.EntityKey_pb2 import EntityKey as EntityKeyProto
from feast.protos.feast.types.Value_pb2 import Value as ValueProto
from feast.repo_config import FeastConfigBaseModel


class FaissOnlineStoreConfig(FeastConfigBaseModel):
    dimension: int
    index_path: str
    index_type: str = "IVFFlat"
    nlist: int = 100


class InMemoryStore:
    def __init__(self):
        self.feature_names: List[str] = []
        self.entity_keys: Dict[str, int] = {}

    def update(self, feature_names: List[str], entity_keys: Dict[str, int]):
        self.feature_names = feature_names
        self.entity_keys = entity_keys

    def delete(self, entity_keys: List[str]):
        for entity_key in entity_keys:
            if entity_key in self.entity_keys:
                del self.entity_keys[entity_key]

    def read(self, entity_keys: List[str]) -> List[Optional[int]]:
        return [self.entity_keys.get(entity_key) for entity_key in entity_keys]

    def teardown(self):
        self.feature_names = []
        self.entity_keys = {}


class FaissOnlineStore(OnlineStore):
    _index: Optional[faiss.IndexIVFFlat] = None
    _in_memory_store: InMemoryStore = InMemoryStore()
    _config: Optional[FaissOnlineStoreConfig] = None
    _logger: logging.Logger = logging.getLogger(__name__)

    def _get_index(self, config: RepoConfig) -> faiss.IndexIVFFlat:
        if self._index is None or self._config is None:
            raise ValueError("Index is not initialized")
        return self._index

    def update(
        self,
        config: RepoConfig,
        tables_to_delete: Sequence[FeatureView],
        tables_to_keep: Sequence[FeatureView],
        entities_to_delete: Sequence[Entity],
        entities_to_keep: Sequence[Entity],
        partial: bool,
    ):
        feature_views = tables_to_keep
        if not feature_views:
            return

        feature_names = [f.name for f in feature_views[0].features]
        dimension = len(feature_names)

        self._config = FaissOnlineStoreConfig(**config.online_store.dict())
        if self._index is None or not partial:
            quantizer = faiss.IndexFlatL2(dimension)
            self._index = faiss.IndexIVFFlat(quantizer, dimension, self._config.nlist)
            self._index.train(
                np.random.rand(self._config.nlist * 100, dimension).astype(np.float32)
            )
            self._in_memory_store = InMemoryStore()

        self._in_memory_store.update(feature_names, {})

    def teardown(
        self,
        config: RepoConfig,
        tables: Sequence[FeatureView],
        entities: Sequence[Entity],
    ):
        self._index = None
        self._in_memory_store.teardown()

    def online_read(
        self,
        config: RepoConfig,
        table: FeatureView,
        entity_keys: List[EntityKeyProto],
        requested_features: Optional[List[str]] = None,
    ) -> List[Tuple[Optional[datetime], Optional[Dict[str, ValueProto]]]]:
        if self._index is None:
            return [(None, None)] * len(entity_keys)

        results: List[Tuple[Optional[datetime], Optional[Dict[str, Any]]]] = []
        for entity_key in entity_keys:
            serialized_key = serialize_entity_key(
                entity_key, config.entity_key_serialization_version
            ).hex()
            idx = self._in_memory_store.entity_keys.get(serialized_key, -1)
            if idx == -1:
                results.append((None, None))
            else:
                feature_vector = self._index.reconstruct(int(idx))
                feature_dict = {
                    name: ValueProto(double_val=value)
                    for name, value in zip(
                        self._in_memory_store.feature_names, feature_vector
                    )
                }
                results.append((None, feature_dict))
        return results

    def online_write_batch(
        self,
        config: RepoConfig,
        table: FeatureView,
        data: List[
            Tuple[EntityKeyProto, Dict[str, ValueProto], datetime, Optional[datetime]]
        ],
        progress: Optional[Callable[[int], Any]],
    ) -> None:
        if self._index is None:
            self._logger.warning("Index is not initialized. Skipping write operation.")
            return

        feature_vectors = []
        serialized_keys = []

        for entity_key, feature_dict, _, _ in data:
            serialized_key = serialize_entity_key(
                entity_key, config.entity_key_serialization_version
            ).hex()
            feature_vector = np.array(
                [
                    feature_dict[name].double_val
                    for name in self._in_memory_store.feature_names
                ],
                dtype=np.float32,
            )

            feature_vectors.append(feature_vector)
            serialized_keys.append(serialized_key)

        feature_vectors_array = np.array(feature_vectors)

        existing_indices = [
            self._in_memory_store.entity_keys.get(sk, -1) for sk in serialized_keys
        ]
        mask = np.array(existing_indices) != -1
        if np.any(mask):
            self._index.remove_ids(
                np.array([idx for idx in existing_indices if idx != -1])
            )

        new_indices = np.arange(
            self._index.ntotal, self._index.ntotal + len(feature_vectors_array)
        )
        self._index.add(feature_vectors_array)

        for sk, idx in zip(serialized_keys, new_indices):
            self._in_memory_store.entity_keys[sk] = idx

        if progress:
            progress(len(data))

    def retrieve_online_documents(
        self,
        config: RepoConfig,
        table: FeatureView,
        requested_featres: List[str],
        embedding: List[float],
        top_k: int,
        distance_metric: Optional[str] = None,
    ) -> List[
        Tuple[
            Optional[datetime],
            Optional[EntityKeyProto],
            Optional[ValueProto],
            Optional[ValueProto],
            Optional[ValueProto],
        ]
    ]:
        if self._index is None:
            self._logger.warning("Index is not initialized. Returning empty result.")
            return []

        query_vector = np.array(embedding, dtype=np.float32).reshape(1, -1)
        distances, indices = self._index.search(query_vector, top_k)

        results: List[
            Tuple[
                Optional[datetime],
                Optional[EntityKeyProto],
                Optional[ValueProto],
                Optional[ValueProto],
                Optional[ValueProto],
            ]
        ] = []
        for i, idx in enumerate(indices[0]):
            if idx == -1:
                continue

            feature_vector = self._index.reconstruct(int(idx))

            timestamp = Timestamp()
            timestamp.GetCurrentTime()
            entity_value = EntityKeyProto()
            feature_value = ValueProto(string_val=",".join(map(str, feature_vector)))
            vector_value = ValueProto(string_val=",".join(map(str, feature_vector)))
            distance_value = ValueProto(float_val=distances[0][i])

            results.append(
                (
                    timestamp.ToDatetime(),
                    entity_value,
                    feature_value,
                    vector_value,
                    distance_value,
                )
            )

        return results

    async def online_read_async(
        self,
        config: RepoConfig,
        table: FeatureView,
        entity_keys: List[EntityKeyProto],
        requested_features: Optional[List[str]] = None,
    ) -> List[Tuple[Optional[datetime], Optional[Dict[str, ValueProto]]]]:
        # Implement async read if needed
        raise NotImplementedError("Async read is not implemented for FaissOnlineStore")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/serving/ServingService.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from feast.protos.feast.types import Value_pb2 as feast_dot_types_dot_Value__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"feast/serving/ServingService.proto\x12\rfeast.serving\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x17\x66\x65\x61st/types/Value.proto\"\x1c\n\x1aGetFeastServingInfoRequest\".\n\x1bGetFeastServingInfoResponse\x12\x0f\n\x07version\x18\x01 \x01(\t\"E\n\x12\x46\x65\x61tureReferenceV2\x12\x19\n\x11\x66\x65\x61ture_view_name\x18\x01 \x01(\t\x12\x14\n\x0c\x66\x65\x61ture_name\x18\x02 \x01(\t\"\xfd\x02\n\x1aGetOnlineFeaturesRequestV2\x12\x33\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32!.feast.serving.FeatureReferenceV2\x12H\n\x0b\x65ntity_rows\x18\x02 \x03(\x0b\x32\x33.feast.serving.GetOnlineFeaturesRequestV2.EntityRow\x12\x0f\n\x07project\x18\x05 \x01(\t\x1a\xce\x01\n\tEntityRow\x12-\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12O\n\x06\x66ields\x18\x02 \x03(\x0b\x32?.feast.serving.GetOnlineFeaturesRequestV2.EntityRow.FieldsEntry\x1a\x41\n\x0b\x46ieldsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.feast.types.Value:\x02\x38\x01\"\x1a\n\x0b\x46\x65\x61tureList\x12\x0b\n\x03val\x18\x01 \x03(\t\"\xc8\x03\n\x18GetOnlineFeaturesRequest\x12\x19\n\x0f\x66\x65\x61ture_service\x18\x01 \x01(\tH\x00\x12.\n\x08\x66\x65\x61tures\x18\x02 \x01(\x0b\x32\x1a.feast.serving.FeatureListH\x00\x12G\n\x08\x65ntities\x18\x03 \x03(\x0b\x32\x35.feast.serving.GetOnlineFeaturesRequest.EntitiesEntry\x12\x1a\n\x12\x66ull_feature_names\x18\x04 \x01(\x08\x12T\n\x0frequest_context\x18\x05 \x03(\x0b\x32;.feast.serving.GetOnlineFeaturesRequest.RequestContextEntry\x1aK\n\rEntitiesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.feast.types.RepeatedValue:\x02\x38\x01\x1aQ\n\x13RequestContextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\x05value\x18\x02 \x01(\x0b\x32\x1a.feast.types.RepeatedValue:\x02\x38\x01\x42\x06\n\x04kind\"\xd2\x02\n\x19GetOnlineFeaturesResponse\x12\x42\n\x08metadata\x18\x01 \x01(\x0b\x32\x30.feast.serving.GetOnlineFeaturesResponseMetadata\x12G\n\x07results\x18\x02 \x03(\x0b\x32\x36.feast.serving.GetOnlineFeaturesResponse.FeatureVector\x12\x0e\n\x06status\x18\x03 \x01(\x08\x1a\x97\x01\n\rFeatureVector\x12\"\n\x06values\x18\x01 \x03(\x0b\x32\x12.feast.types.Value\x12,\n\x08statuses\x18\x02 \x03(\x0e\x32\x1a.feast.serving.FieldStatus\x12\x34\n\x10\x65vent_timestamps\x18\x03 \x03(\x0b\x32\x1a.google.protobuf.Timestamp\"V\n!GetOnlineFeaturesResponseMetadata\x12\x31\n\rfeature_names\x18\x01 \x01(\x0b\x32\x1a.feast.serving.FeatureList*[\n\x0b\x46ieldStatus\x12\x0b\n\x07INVALID\x10\x00\x12\x0b\n\x07PRESENT\x10\x01\x12\x0e\n\nNULL_VALUE\x10\x02\x12\r\n\tNOT_FOUND\x10\x03\x12\x13\n\x0fOUTSIDE_MAX_AGE\x10\x04\x32\xe6\x01\n\x0eServingService\x12l\n\x13GetFeastServingInfo\x12).feast.serving.GetFeastServingInfoRequest\x1a*.feast.serving.GetFeastServingInfoResponse\x12\x66\n\x11GetOnlineFeatures\x12\'.feast.serving.GetOnlineFeaturesRequest\x1a(.feast.serving.GetOnlineFeaturesResponseBZ\n\x13\x66\x65\x61st.proto.servingB\x0fServingAPIProtoZ2github.com/feast-dev/feast/go/protos/feast/servingb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.serving.ServingService_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\023feast.proto.servingB\017ServingAPIProtoZ2github.com/feast-dev/feast/go/protos/feast/serving'
  _globals['_GETONLINEFEATURESREQUESTV2_ENTITYROW_FIELDSENTRY']._options = None
  _globals['_GETONLINEFEATURESREQUESTV2_ENTITYROW_FIELDSENTRY']._serialized_options = b'8\001'
  _globals['_GETONLINEFEATURESREQUEST_ENTITIESENTRY']._options = None
  _globals['_GETONLINEFEATURESREQUEST_ENTITIESENTRY']._serialized_options = b'8\001'
  _globals['_GETONLINEFEATURESREQUEST_REQUESTCONTEXTENTRY']._options = None
  _globals['_GETONLINEFEATURESREQUEST_REQUESTCONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_FIELDSTATUS']._serialized_start=1560
  _globals['_FIELDSTATUS']._serialized_end=1651
  _globals['_GETFEASTSERVINGINFOREQUEST']._serialized_start=111
  _globals['_GETFEASTSERVINGINFOREQUEST']._serialized_end=139
  _globals['_GETFEASTSERVINGINFORESPONSE']._serialized_start=141
  _globals['_GETFEASTSERVINGINFORESPONSE']._serialized_end=187
  _globals['_FEATUREREFERENCEV2']._serialized_start=189
  _globals['_FEATUREREFERENCEV2']._serialized_end=258
  _globals['_GETONLINEFEATURESREQUESTV2']._serialized_start=261
  _globals['_GETONLINEFEATURESREQUESTV2']._serialized_end=642
  _globals['_GETONLINEFEATURESREQUESTV2_ENTITYROW']._serialized_start=436
  _globals['_GETONLINEFEATURESREQUESTV2_ENTITYROW']._serialized_end=642
  _globals['_GETONLINEFEATURESREQUESTV2_ENTITYROW_FIELDSENTRY']._serialized_start=577
  _globals['_GETONLINEFEATURESREQUESTV2_ENTITYROW_FIELDSENTRY']._serialized_end=642
  _globals['_FEATURELIST']._serialized_start=644
  _globals['_FEATURELIST']._serialized_end=670
  _globals['_GETONLINEFEATURESREQUEST']._serialized_start=673
  _globals['_GETONLINEFEATURESREQUEST']._serialized_end=1129
  _globals['_GETONLINEFEATURESREQUEST_ENTITIESENTRY']._serialized_start=963
  _globals['_GETONLINEFEATURESREQUEST_ENTITIESENTRY']._serialized_end=1038
  _globals['_GETONLINEFEATURESREQUEST_REQUESTCONTEXTENTRY']._serialized_start=1040
  _globals['_GETONLINEFEATURESREQUEST_REQUESTCONTEXTENTRY']._serialized_end=1121
  _globals['_GETONLINEFEATURESRESPONSE']._serialized_start=1132
  _globals['_GETONLINEFEATURESRESPONSE']._serialized_end=1470
  _globals['_GETONLINEFEATURESRESPONSE_FEATUREVECTOR']._serialized_start=1319
  _globals['_GETONLINEFEATURESRESPONSE_FEATUREVECTOR']._serialized_end=1470
  _globals['_GETONLINEFEATURESRESPONSEMETADATA']._serialized_start=1472
  _globals['_GETONLINEFEATURESRESPONSEMETADATA']._serialized_end=1558
  _globals['_SERVINGSERVICE']._serialized_start=1654
  _globals['_SERVINGSERVICE']._serialized_end=1884
# @@protoc_insertion_point(module_scope)



import json
from datetime import timedelta
from typing import Any, Callable, Dict, Iterable, Optional, Tuple

from couchbase_columnar.cluster import Cluster
from couchbase_columnar.credential import Credential
from couchbase_columnar.options import ClusterOptions, QueryOptions, TimeoutOptions
from typeguard import typechecked

from feast.data_source import DataSource
from feast.errors import DataSourceNoNameException, ZeroColumnQueryResult
from feast.feature_logging import LoggingDestination
from feast.protos.feast.core.DataSource_pb2 import DataSource as DataSourceProto
from feast.protos.feast.core.FeatureService_pb2 import (
    LoggingConfig as LoggingConfigProto,
)
from feast.protos.feast.core.SavedDataset_pb2 import (
    SavedDatasetStorage as SavedDatasetStorageProto,
)
from feast.repo_config import RepoConfig
from feast.saved_dataset import SavedDatasetStorage
from feast.type_map import ValueType, cb_columnar_type_to_feast_value_type


@typechecked
class CouchbaseColumnarSource(DataSource):
    """A CouchbaseColumnarSource object defines a data source that a CouchbaseColumnarOfflineStore class can use."""

    def source_type(self) -> DataSourceProto.SourceType.ValueType:
        # TODO: Add Couchbase to DataSourceProto.SourceType
        return DataSourceProto.CUSTOM_SOURCE

    def __init__(
        self,
        name: Optional[str] = None,
        query: Optional[str] = None,
        database: Optional[str] = "Default",
        scope: Optional[str] = "Default",
        collection: Optional[str] = None,
        timestamp_field: Optional[str] = "",
        created_timestamp_column: Optional[str] = "",
        field_mapping: Optional[Dict[str, str]] = None,
        description: Optional[str] = "",
        tags: Optional[Dict[str, str]] = None,
        owner: Optional[str] = "",
    ):
        """Creates a CouchbaseColumnarSource object.

        Args:
            name: Name of CouchbaseColumnarSource, which should be unique within a project.
            query: SQL++ query that will be used to fetch the data.
            database: Columnar database name.
            scope: Columnar scope name.
            collection: Columnar collection name.
            timestamp_field (optional): Event timestamp field used for point-in-time joins of
                feature values.
            created_timestamp_column (optional): Timestamp column indicating when the row
                was created, used for deduplicating rows.
            field_mapping (optional): A dictionary mapping of field names in this data
                source to feature names in a feature table or view. Only used for feature
                fields, not entity or timestamp fields.
            description (optional): A human-readable description.
            tags (optional): A dictionary of key-value pairs to store arbitrary metadata.
            owner (optional): The owner of the data source, typically the email of the primary
                maintainer.
        """
        self._couchbase_options = CouchbaseColumnarOptions(
            name=name,
            query=query,
            database=database,
            scope=scope,
            collection=collection,
        )

        # If no name, use the collection as the default name.
        if name is None and collection is None:
            raise DataSourceNoNameException()
        name = name or collection
        assert name

        super().__init__(
            name=name,
            timestamp_field=timestamp_field,
            created_timestamp_column=created_timestamp_column,
            field_mapping=field_mapping,
            description=description,
            tags=tags,
            owner=owner,
        )

    def __hash__(self):
        return super().__hash__()

    def __eq__(self, other):
        if not isinstance(other, CouchbaseColumnarSource):
            raise TypeError(
                "Comparisons should only involve CouchbaseColumnarSource class objects."
            )

        return (
            super().__eq__(other)
            and self._couchbase_options._query == other._couchbase_options._query
            and self.timestamp_field == other.timestamp_field
            and self.created_timestamp_column == other.created_timestamp_column
            and self.field_mapping == other.field_mapping
        )

    @staticmethod
    def from_proto(data_source: DataSourceProto):
        assert data_source.HasField("custom_options")

        couchbase_options = json.loads(data_source.custom_options.configuration)

        return CouchbaseColumnarSource(
            name=couchbase_options["name"],
            query=couchbase_options["query"],
            database=couchbase_options["database"],
            scope=couchbase_options["scope"],
            collection=couchbase_options["collection"],
            field_mapping=dict(data_source.field_mapping),
            timestamp_field=data_source.timestamp_field,
            created_timestamp_column=data_source.created_timestamp_column,
            description=data_source.description,
            tags=dict(data_source.tags),
            owner=data_source.owner,
        )

    def _to_proto_impl(self) -> DataSourceProto:
        data_source_proto = DataSourceProto(
            name=self.name,
            type=DataSourceProto.CUSTOM_SOURCE,
            data_source_class_type="feast.infra.offline_stores.contrib.couchbase_offline_store.couchbase_source.CouchbaseColumnarSource",
            field_mapping=self.field_mapping,
            custom_options=self._couchbase_options.to_proto(),
            description=self.description,
            tags=self.tags,
            owner=self.owner,
        )

        data_source_proto.timestamp_field = self.timestamp_field
        data_source_proto.created_timestamp_column = self.created_timestamp_column

        return data_source_proto

    def validate(self, config: RepoConfig):
        pass

    @staticmethod
    def source_datatype_to_feast_value_type() -> Callable[[str], ValueType]:
        # Define the type conversion for Couchbase fields to Feast ValueType as needed
        return cb_columnar_type_to_feast_value_type

    def _infer_composite_type(self, field: Dict[str, Any]) -> str:
        """
        Infers type signature for a field, rejecting complex nested structures that
        aren't compatible with Feast's type system.

        Args:
            field: Dictionary containing field information including type and nested structures

        Returns:
            String representation of the type, or raises ValueError for incompatible types

        Raises:
            ValueError: If field contains complex nested structures not supported by Feast
        """
        base_type = field.get("field-type", "unknown").lower()

        if base_type == "array":
            if "list" not in field or not field["list"]:
                return "array<unknown>"

            item_type = field["list"][0]
            if item_type.get("field-type") == "object":
                raise ValueError(
                    "Complex object types in arrays are not supported by Feast. "
                    "Arrays must contain homogeneous primitive values."
                )

            # Only allow arrays of primitive types
            inner_type = item_type.get("field-type", "unknown")
            if inner_type in ["array", "multiset", "object"]:
                raise ValueError(
                    "Nested collection types are not supported by Feast. "
                    "Arrays can only be one level deep."
                )

            return f"array<{inner_type}>"

        elif base_type == "object":
            raise ValueError(
                "Complex object types are not supported by Feast. "
                "Only primitive types and homogeneous arrays are allowed."
            )

        elif base_type == "multiset":
            raise ValueError(
                "Multiset types are not supported by Feast. "
                "Only primitive types and homogeneous arrays are allowed."
            )

        return base_type

    def get_table_column_names_and_types(
        self, config: RepoConfig
    ) -> Iterable[Tuple[str, str]]:
        cred = Credential.from_username_and_password(
            config.offline_store.user, config.offline_store.password
        )
        timeout_opts = TimeoutOptions(dispatch_timeout=timedelta(seconds=120))
        cluster = Cluster.create_instance(
            config.offline_store.connection_string,
            cred,
            ClusterOptions(timeout_options=timeout_opts),
        )

        query_context = self.get_table_query_string()
        query = f"""
        SELECT get_object_fields(
            CASE WHEN ARRAY_LENGTH(OBJECT_PAIRS(t)) = 1 AND OBJECT_PAIRS(t)[0].`value` IS NOT MISSING
                 THEN OBJECT_PAIRS(t)[0].`value`
                 ELSE t
            END
        ) AS field_types
        FROM {query_context} AS t
        LIMIT 1;
        """

        result = cluster.execute_query(
            query, QueryOptions(timeout=timedelta(seconds=config.offline_store.timeout))
        )
        if not result:
            raise ZeroColumnQueryResult(query)

        rows = result.get_all_rows()
        field_type_pairs = []
        if rows and rows[0]:
            # Accessing the "field_types" array from the first row
            field_types_list = rows[0].get("field_types", [])
            for field in field_types_list:
                field_name = field.get("field-name", "unknown")
                field_type = field.get("field-type", "unknown")
                # drop uuid fields to ensure schema matches dataframe
                if field_type == "uuid":
                    continue
                field_type = self._infer_composite_type(field)
                field_type_pairs.append((field_name, field_type))
        return field_type_pairs

    def get_table_query_string(self) -> str:
        if (
            self._couchbase_options._database
            and self._couchbase_options._scope
            and self._couchbase_options._collection
        ):
            return f"`{self._couchbase_options._database}`.`{self._couchbase_options._scope}`.`{self._couchbase_options._collection}`"
        else:
            return f"({self._couchbase_options._query})"

    @property
    def database(self) -> str:
        """Returns the database name."""
        return self._couchbase_options._database

    @property
    def scope(self) -> str:
        """Returns the scope name."""
        return self._couchbase_options._scope


class CouchbaseColumnarOptions:
    def __init__(
        self,
        name: Optional[str],
        query: Optional[str],
        database: Optional[str],
        scope: Optional[str],
        collection: Optional[str],
    ):
        self._name = name or ""
        self._query = query or ""
        self._database = database or ""
        self._scope = scope or ""
        self._collection = collection or ""

    @classmethod
    def from_proto(cls, couchbase_options_proto: DataSourceProto.CustomSourceOptions):
        config = json.loads(couchbase_options_proto.configuration.decode("utf8"))
        couchbase_options = cls(
            name=config["name"],
            query=config["query"],
            database=config["database"],
            scope=config["scope"],
            collection=config["collection"],
        )

        return couchbase_options

    def to_proto(self) -> DataSourceProto.CustomSourceOptions:
        couchbase_options_proto = DataSourceProto.CustomSourceOptions(
            configuration=json.dumps(
                {
                    "name": self._name,
                    "query": self._query,
                    "database": self._database,
                    "scope": self._scope,
                    "collection": self._collection,
                }
            ).encode()
        )
        return couchbase_options_proto


class SavedDatasetCouchbaseColumnarStorage(SavedDatasetStorage):
    _proto_attr_name = "custom_storage"

    couchbase_options: CouchbaseColumnarOptions

    def __init__(self, database_ref: str, scope_ref: str, collection_ref: str):
        self.couchbase_options = CouchbaseColumnarOptions(
            database=database_ref,
            scope=scope_ref,
            collection=collection_ref,
            name=None,
            query=None,
        )

    @staticmethod
    def from_proto(storage_proto: SavedDatasetStorageProto) -> SavedDatasetStorage:
        return SavedDatasetCouchbaseColumnarStorage(
            database_ref=CouchbaseColumnarOptions.from_proto(
                storage_proto.custom_storage
            )._database,
            scope_ref=CouchbaseColumnarOptions.from_proto(
                storage_proto.custom_storage
            )._scope,
            collection_ref=CouchbaseColumnarOptions.from_proto(
                storage_proto.custom_storage
            )._collection,
        )

    def to_proto(self) -> SavedDatasetStorageProto:
        return SavedDatasetStorageProto(
            custom_storage=self.couchbase_options.to_proto()
        )

    def to_data_source(self) -> DataSource:
        return CouchbaseColumnarSource(
            database=self.couchbase_options._database,
            scope=self.couchbase_options._scope,
            collection=self.couchbase_options._collection,
        )


class CouchbaseColumnarLoggingDestination(LoggingDestination):
    """
    Couchbase Columnar implementation of a logging destination.
    """

    database: str
    scope: str
    table_name: str

    _proto_kind = "couchbase_columnar_destination"

    def __init__(self, *, database: str, scope: str, table_name: str):
        """
        Args:
            database: The Couchbase database name
            scope: The Couchbase scope name
            table_name: The Couchbase collection name to log features into
        """
        self.database = database
        self.scope = scope
        self.table_name = table_name

    def to_data_source(self) -> DataSource:
        """
        Returns a data source object representing the logging destination.
        """
        return CouchbaseColumnarSource(
            database=self.database,
            scope=self.scope,
            collection=self.table_name,
        )

    def to_proto(self) -> LoggingConfigProto:
        """
        Converts the logging destination to its protobuf representation.
        """
        return LoggingConfigProto(
            couchbase_columnar_destination=LoggingConfigProto.CouchbaseColumnarDestination(
                database=self.database,
                scope=self.scope,
                collection=self.table_name,
            )
        )

    @classmethod
    def from_proto(
        cls, config_proto: LoggingConfigProto
    ) -> "CouchbaseColumnarLoggingDestination":
        """
        Creates a CouchbaseColumnarLoggingDestination from its protobuf representation.
        """
        return CouchbaseColumnarLoggingDestination(
            database=config_proto.CouchbaseColumnarDestination.database,
            scope=config_proto.CouchbaseColumnarDestination.scope,
            table_name=config_proto.CouchbaseColumnarDestination.collection,
        )



import atexit
import logging
import threading
import warnings
from abc import abstractmethod
from datetime import timedelta
from threading import Lock
from typing import List, Optional

from feast.base_feature_view import BaseFeatureView
from feast.data_source import DataSource
from feast.entity import Entity
from feast.feature_service import FeatureService
from feast.feature_view import FeatureView
from feast.infra.infra_object import Infra
from feast.infra.registry import proto_registry_utils
from feast.infra.registry.base_registry import BaseRegistry
from feast.on_demand_feature_view import OnDemandFeatureView
from feast.permissions.permission import Permission
from feast.project import Project
from feast.project_metadata import ProjectMetadata
from feast.protos.feast.core.Registry_pb2 import Registry as RegistryProto
from feast.saved_dataset import SavedDataset, ValidationReference
from feast.stream_feature_view import StreamFeatureView
from feast.utils import _utc_now

logger = logging.getLogger(__name__)


class CachingRegistry(BaseRegistry):
    def __init__(self, project: str, cache_ttl_seconds: int, cache_mode: str):
        self.cache_mode = cache_mode
        self.cached_registry_proto = RegistryProto()
        self._refresh_lock = Lock()
        self.cached_registry_proto_ttl = timedelta(
            seconds=cache_ttl_seconds if cache_ttl_seconds is not None else 0
        )
        self.cached_registry_proto = self.proto()
        self.cached_registry_proto_created = _utc_now()
        if cache_mode == "thread":
            self._start_thread_async_refresh(cache_ttl_seconds)
            atexit.register(self._exit_handler)

    @abstractmethod
    def _get_data_source(self, name: str, project: str) -> DataSource:
        pass

    def get_data_source(
        self, name: str, project: str, allow_cache: bool = False
    ) -> DataSource:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_data_source(
                self.cached_registry_proto, name, project
            )
        return self._get_data_source(name, project)

    @abstractmethod
    def _list_data_sources(
        self, project: str, tags: Optional[dict[str, str]]
    ) -> List[DataSource]:
        pass

    def list_data_sources(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[DataSource]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_data_sources(
                self.cached_registry_proto, project, tags
            )
        return self._list_data_sources(project, tags)

    @abstractmethod
    def _get_entity(self, name: str, project: str) -> Entity:
        pass

    def get_entity(self, name: str, project: str, allow_cache: bool = False) -> Entity:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_entity(
                self.cached_registry_proto, name, project
            )
        return self._get_entity(name, project)

    @abstractmethod
    def _list_entities(
        self, project: str, tags: Optional[dict[str, str]]
    ) -> List[Entity]:
        pass

    def list_entities(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[Entity]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_entities(
                self.cached_registry_proto, project, tags
            )
        return self._list_entities(project, tags)

    @abstractmethod
    def _get_any_feature_view(self, name: str, project: str) -> BaseFeatureView:
        pass

    def get_any_feature_view(
        self, name: str, project: str, allow_cache: bool = False
    ) -> BaseFeatureView:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_any_feature_view(
                self.cached_registry_proto, name, project
            )
        return self._get_any_feature_view(name, project)

    @abstractmethod
    def _list_all_feature_views(
        self, project: str, tags: Optional[dict[str, str]]
    ) -> List[BaseFeatureView]:
        pass

    def list_all_feature_views(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[BaseFeatureView]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_all_feature_views(
                self.cached_registry_proto, project, tags
            )
        return self._list_all_feature_views(project, tags)

    @abstractmethod
    def _get_feature_view(self, name: str, project: str) -> FeatureView:
        pass

    def get_feature_view(
        self, name: str, project: str, allow_cache: bool = False
    ) -> FeatureView:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_feature_view(
                self.cached_registry_proto, name, project
            )
        return self._get_feature_view(name, project)

    @abstractmethod
    def _list_feature_views(
        self, project: str, tags: Optional[dict[str, str]]
    ) -> List[FeatureView]:
        pass

    def list_feature_views(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[FeatureView]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_feature_views(
                self.cached_registry_proto, project, tags
            )
        return self._list_feature_views(project, tags)

    @abstractmethod
    def _get_on_demand_feature_view(
        self, name: str, project: str
    ) -> OnDemandFeatureView:
        pass

    def get_on_demand_feature_view(
        self, name: str, project: str, allow_cache: bool = False
    ) -> OnDemandFeatureView:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_on_demand_feature_view(
                self.cached_registry_proto, name, project
            )
        return self._get_on_demand_feature_view(name, project)

    @abstractmethod
    def _list_on_demand_feature_views(
        self, project: str, tags: Optional[dict[str, str]]
    ) -> List[OnDemandFeatureView]:
        pass

    def list_on_demand_feature_views(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[OnDemandFeatureView]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_on_demand_feature_views(
                self.cached_registry_proto, project, tags
            )
        return self._list_on_demand_feature_views(project, tags)

    @abstractmethod
    def _get_stream_feature_view(self, name: str, project: str) -> StreamFeatureView:
        pass

    def get_stream_feature_view(
        self, name: str, project: str, allow_cache: bool = False
    ) -> StreamFeatureView:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_stream_feature_view(
                self.cached_registry_proto, name, project
            )
        return self._get_stream_feature_view(name, project)

    @abstractmethod
    def _list_stream_feature_views(
        self, project: str, tags: Optional[dict[str, str]]
    ) -> List[StreamFeatureView]:
        pass

    def list_stream_feature_views(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[StreamFeatureView]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_stream_feature_views(
                self.cached_registry_proto, project, tags
            )
        return self._list_stream_feature_views(project, tags)

    @abstractmethod
    def _get_feature_service(self, name: str, project: str) -> FeatureService:
        pass

    def get_feature_service(
        self, name: str, project: str, allow_cache: bool = False
    ) -> FeatureService:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_feature_service(
                self.cached_registry_proto, name, project
            )
        return self._get_feature_service(name, project)

    @abstractmethod
    def _list_feature_services(
        self, project: str, tags: Optional[dict[str, str]]
    ) -> List[FeatureService]:
        pass

    def list_feature_services(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[FeatureService]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_feature_services(
                self.cached_registry_proto, project, tags
            )
        return self._list_feature_services(project, tags)

    @abstractmethod
    def _get_saved_dataset(self, name: str, project: str) -> SavedDataset:
        pass

    def get_saved_dataset(
        self, name: str, project: str, allow_cache: bool = False
    ) -> SavedDataset:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_saved_dataset(
                self.cached_registry_proto, name, project
            )
        return self._get_saved_dataset(name, project)

    @abstractmethod
    def _list_saved_datasets(
        self, project: str, tags: Optional[dict[str, str]] = None
    ) -> List[SavedDataset]:
        pass

    def list_saved_datasets(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[SavedDataset]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_saved_datasets(
                self.cached_registry_proto, project, tags
            )
        return self._list_saved_datasets(project, tags)

    @abstractmethod
    def _get_validation_reference(self, name: str, project: str) -> ValidationReference:
        pass

    def get_validation_reference(
        self, name: str, project: str, allow_cache: bool = False
    ) -> ValidationReference:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_validation_reference(
                self.cached_registry_proto, name, project
            )
        return self._get_validation_reference(name, project)

    @abstractmethod
    def _list_validation_references(
        self, project: str, tags: Optional[dict[str, str]] = None
    ) -> List[ValidationReference]:
        pass

    def list_validation_references(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[ValidationReference]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_validation_references(
                self.cached_registry_proto, project, tags
            )
        return self._list_validation_references(project, tags)

    @abstractmethod
    def _list_project_metadata(self, project: str) -> List[ProjectMetadata]:
        pass

    def list_project_metadata(
        self, project: str, allow_cache: bool = False
    ) -> List[ProjectMetadata]:
        warnings.warn(
            "list_project_metadata is deprecated and will be removed in a future version. Use list_projects() and get_project() methods instead.",
            DeprecationWarning,
        )
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_project_metadata(
                self.cached_registry_proto, project
            )
        return self._list_project_metadata(project)

    @abstractmethod
    def _get_infra(self, project: str) -> Infra:
        pass

    def get_infra(self, project: str, allow_cache: bool = False) -> Infra:
        return self._get_infra(project)

    @abstractmethod
    def _get_permission(self, name: str, project: str) -> Permission:
        pass

    def get_permission(
        self, name: str, project: str, allow_cache: bool = False
    ) -> Permission:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_permission(
                self.cached_registry_proto, name, project
            )
        return self._get_permission(name, project)

    @abstractmethod
    def _list_permissions(
        self, project: str, tags: Optional[dict[str, str]]
    ) -> List[Permission]:
        pass

    def list_permissions(
        self,
        project: str,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[Permission]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_permissions(
                self.cached_registry_proto, project, tags
            )
        return self._list_permissions(project, tags)

    @abstractmethod
    def _get_project(self, name: str) -> Project:
        pass

    def get_project(
        self,
        name: str,
        allow_cache: bool = False,
    ) -> Project:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.get_project(self.cached_registry_proto, name)
        return self._get_project(name)

    @abstractmethod
    def _list_projects(self, tags: Optional[dict[str, str]]) -> List[Project]:
        pass

    def list_projects(
        self,
        allow_cache: bool = False,
        tags: Optional[dict[str, str]] = None,
    ) -> List[Project]:
        if allow_cache:
            self._refresh_cached_registry_if_necessary()
            return proto_registry_utils.list_projects(self.cached_registry_proto, tags)
        return self._list_projects(tags)

    def refresh(self, project: Optional[str] = None):
        try:
            self.cached_registry_proto = self.proto()
            self.cached_registry_proto_created = _utc_now()
        except Exception as e:
            logger.debug(f"Error while refreshing registry: {e}", exc_info=True)

    def _refresh_cached_registry_if_necessary(self):
        if self.cache_mode == "sync":

            def is_cache_expired():
                if (
                    self.cached_registry_proto is None
                    or self.cached_registry_proto == RegistryProto()
                ):
                    return True

                # Cache is expired if creation time is None
                if (
                    not hasattr(self, "cached_registry_proto_created")
                    or self.cached_registry_proto_created is None
                ):
                    return True

                # Cache is expired if TTL > 0 and current time exceeds creation + TTL
                if self.cached_registry_proto_ttl.total_seconds() > 0 and _utc_now() > (
                    self.cached_registry_proto_created + self.cached_registry_proto_ttl
                ):
                    return True

                return False

            if is_cache_expired():
                if not self._refresh_lock.acquire(blocking=False):
                    logger.debug(
                        "Skipping refresh if lock is already held by another thread"
                    )
                    return
                try:
                    logger.info(
                        f"Registry cache expired(ttl: {self.cached_registry_proto_ttl.total_seconds()} seconds), so refreshing"
                    )
                    self.refresh()
                except Exception as e:
                    logger.debug(
                        f"Error in _refresh_cached_registry_if_necessary: {e}",
                        exc_info=True,
                    )
                finally:
                    self._refresh_lock.release()

    def _start_thread_async_refresh(self, cache_ttl_seconds):
        self.refresh()
        if cache_ttl_seconds <= 0:
            return
        self.registry_refresh_thread = threading.Timer(
            cache_ttl_seconds, self._start_thread_async_refresh, [cache_ttl_seconds]
        )
        self.registry_refresh_thread.daemon = True
        self.registry_refresh_thread.start()

    def _exit_handler(self):
        self.registry_refresh_thread.cancel()



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file

Copyright 2020 The Feast Authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import builtins
import collections.abc
import feast.core.Aggregation_pb2
import feast.core.DataSource_pb2
import feast.core.FeatureView_pb2
import feast.core.Feature_pb2
import feast.core.OnDemandFeatureView_pb2
import feast.core.Transformation_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class StreamFeatureView(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SPEC_FIELD_NUMBER: builtins.int
    META_FIELD_NUMBER: builtins.int
    @property
    def spec(self) -> global___StreamFeatureViewSpec:
        """User-specified specifications of this feature view."""
    @property
    def meta(self) -> feast.core.FeatureView_pb2.FeatureViewMeta: ...
    def __init__(
        self,
        *,
        spec: global___StreamFeatureViewSpec | None = ...,
        meta: feast.core.FeatureView_pb2.FeatureViewMeta | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["meta", b"meta", "spec", b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["meta", b"meta", "spec", b"spec"]) -> None: ...

global___StreamFeatureView = StreamFeatureView

class StreamFeatureViewSpec(google.protobuf.message.Message):
    """Next available id: 17"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    ENTITIES_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    ENTITY_COLUMNS_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    TTL_FIELD_NUMBER: builtins.int
    BATCH_SOURCE_FIELD_NUMBER: builtins.int
    STREAM_SOURCE_FIELD_NUMBER: builtins.int
    ONLINE_FIELD_NUMBER: builtins.int
    USER_DEFINED_FUNCTION_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    AGGREGATIONS_FIELD_NUMBER: builtins.int
    TIMESTAMP_FIELD_FIELD_NUMBER: builtins.int
    FEATURE_TRANSFORMATION_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the feature view. Must be unique. Not updated."""
    project: builtins.str
    """Name of Feast project that this feature view belongs to."""
    @property
    def entities(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """List of names of entities associated with this feature view."""
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Feature_pb2.FeatureSpecV2]:
        """List of specifications for each feature defined as part of this feature view."""
    @property
    def entity_columns(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Feature_pb2.FeatureSpecV2]:
        """List of specifications for each entity defined as part of this feature view."""
    description: builtins.str
    """Description of the feature view."""
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """User defined metadata"""
    owner: builtins.str
    """Owner of the feature view."""
    @property
    def ttl(self) -> google.protobuf.duration_pb2.Duration:
        """Features in this feature view can only be retrieved from online serving
        younger than ttl. Ttl is measured as the duration of time between
        the feature's event timestamp and when the feature is retrieved
        Feature values outside ttl will be returned as unset values and indicated to end user
        """
    @property
    def batch_source(self) -> feast.core.DataSource_pb2.DataSource:
        """Batch/Offline DataSource where this view can retrieve offline feature data."""
    @property
    def stream_source(self) -> feast.core.DataSource_pb2.DataSource:
        """Streaming DataSource from where this view can consume "online" feature data."""
    online: builtins.bool
    """Whether these features should be served online or not"""
    @property
    def user_defined_function(self) -> feast.core.OnDemandFeatureView_pb2.UserDefinedFunction:
        """Serialized function that is encoded in the streamfeatureview"""
    mode: builtins.str
    """Mode of execution"""
    @property
    def aggregations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Aggregation_pb2.Aggregation]:
        """Aggregation definitions"""
    timestamp_field: builtins.str
    """Timestamp field for aggregation"""
    @property
    def feature_transformation(self) -> feast.core.Transformation_pb2.FeatureTransformationV2:
        """Oneof with {user_defined_function, on_demand_substrait_transformation}"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        entities: collections.abc.Iterable[builtins.str] | None = ...,
        features: collections.abc.Iterable[feast.core.Feature_pb2.FeatureSpecV2] | None = ...,
        entity_columns: collections.abc.Iterable[feast.core.Feature_pb2.FeatureSpecV2] | None = ...,
        description: builtins.str = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        owner: builtins.str = ...,
        ttl: google.protobuf.duration_pb2.Duration | None = ...,
        batch_source: feast.core.DataSource_pb2.DataSource | None = ...,
        stream_source: feast.core.DataSource_pb2.DataSource | None = ...,
        online: builtins.bool = ...,
        user_defined_function: feast.core.OnDemandFeatureView_pb2.UserDefinedFunction | None = ...,
        mode: builtins.str = ...,
        aggregations: collections.abc.Iterable[feast.core.Aggregation_pb2.Aggregation] | None = ...,
        timestamp_field: builtins.str = ...,
        feature_transformation: feast.core.Transformation_pb2.FeatureTransformationV2 | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["batch_source", b"batch_source", "feature_transformation", b"feature_transformation", "stream_source", b"stream_source", "ttl", b"ttl", "user_defined_function", b"user_defined_function"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregations", b"aggregations", "batch_source", b"batch_source", "description", b"description", "entities", b"entities", "entity_columns", b"entity_columns", "feature_transformation", b"feature_transformation", "features", b"features", "mode", b"mode", "name", b"name", "online", b"online", "owner", b"owner", "project", b"project", "stream_source", b"stream_source", "tags", b"tags", "timestamp_field", b"timestamp_field", "ttl", b"ttl", "user_defined_function", b"user_defined_function"]) -> None: ...

global___StreamFeatureViewSpec = StreamFeatureViewSpec



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/FeatureView.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2
from feast.protos.feast.core import Feature_pb2 as feast_dot_core_dot_Feature__pb2
from feast.protos.feast.core import Transformation_pb2 as feast_dot_core_dot_Transformation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1c\x66\x65\x61st/core/FeatureView.proto\x12\nfeast.core\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\x1a\x18\x66\x65\x61st/core/Feature.proto\x1a\x1f\x66\x65\x61st/core/Transformation.proto\"c\n\x0b\x46\x65\x61tureView\x12)\n\x04spec\x18\x01 \x01(\x0b\x32\x1b.feast.core.FeatureViewSpec\x12)\n\x04meta\x18\x02 \x01(\x0b\x32\x1b.feast.core.FeatureViewMeta\"\xc6\x04\n\x0f\x46\x65\x61tureViewSpec\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x10\n\x08\x65ntities\x18\x03 \x03(\t\x12+\n\x08\x66\x65\x61tures\x18\x04 \x03(\x0b\x32\x19.feast.core.FeatureSpecV2\x12\x31\n\x0e\x65ntity_columns\x18\x0c \x03(\x0b\x32\x19.feast.core.FeatureSpecV2\x12\x13\n\x0b\x64\x65scription\x18\n \x01(\t\x12\x33\n\x04tags\x18\x05 \x03(\x0b\x32%.feast.core.FeatureViewSpec.TagsEntry\x12\r\n\x05owner\x18\x0b \x01(\t\x12&\n\x03ttl\x18\x06 \x01(\x0b\x32\x19.google.protobuf.Duration\x12,\n\x0c\x62\x61tch_source\x18\x07 \x01(\x0b\x32\x16.feast.core.DataSource\x12-\n\rstream_source\x18\t \x01(\x0b\x32\x16.feast.core.DataSource\x12\x0e\n\x06online\x18\x08 \x01(\x08\x12\x0f\n\x07offline\x18\r \x01(\x08\x12\x31\n\x0csource_views\x18\x0e \x03(\x0b\x32\x1b.feast.core.FeatureViewSpec\x12\x43\n\x16\x66\x65\x61ture_transformation\x18\x0f \x01(\x0b\x32#.feast.core.FeatureTransformationV2\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xcc\x01\n\x0f\x46\x65\x61tureViewMeta\x12\x35\n\x11\x63reated_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16last_updated_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x46\n\x19materialization_intervals\x18\x03 \x03(\x0b\x32#.feast.core.MaterializationInterval\"w\n\x17MaterializationInterval\x12.\n\nstart_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"@\n\x0f\x46\x65\x61tureViewList\x12-\n\x0c\x66\x65\x61tureviews\x18\x01 \x03(\x0b\x32\x17.feast.core.FeatureViewBU\n\x10\x66\x65\x61st.proto.coreB\x10\x46\x65\x61tureViewProtoZ/github.com/feast-dev/feast/go/protos/feast/coreb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.FeatureView_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\020feast.proto.coreB\020FeatureViewProtoZ/github.com/feast-dev/feast/go/protos/feast/core'
  _globals['_FEATUREVIEWSPEC_TAGSENTRY']._options = None
  _globals['_FEATUREVIEWSPEC_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_FEATUREVIEW']._serialized_start=197
  _globals['_FEATUREVIEW']._serialized_end=296
  _globals['_FEATUREVIEWSPEC']._serialized_start=299
  _globals['_FEATUREVIEWSPEC']._serialized_end=881
  _globals['_FEATUREVIEWSPEC_TAGSENTRY']._serialized_start=838
  _globals['_FEATUREVIEWSPEC_TAGSENTRY']._serialized_end=881
  _globals['_FEATUREVIEWMETA']._serialized_start=884
  _globals['_FEATUREVIEWMETA']._serialized_end=1088
  _globals['_MATERIALIZATIONINTERVAL']._serialized_start=1090
  _globals['_MATERIALIZATIONINTERVAL']._serialized_end=1209
  _globals['_FEATUREVIEWLIST']._serialized_start=1211
  _globals['_FEATUREVIEWLIST']._serialized_end=1275
# @@protoc_insertion_point(module_scope)



import click
from couchbase_columnar.cluster import Cluster
from couchbase_columnar.common.errors import InvalidCredentialError
from couchbase_columnar.credential import Credential
from couchbase_columnar.options import ClusterOptions, QueryOptions, TimeoutOptions

from feast.file_utils import replace_str_in_file
from feast.infra.offline_stores.contrib.couchbase_offline_store.couchbase import (
    CouchbaseColumnarOfflineStoreConfig,
    df_to_columnar,
)


def bootstrap():
    # Bootstrap() will automatically be called from the init_repo() during `feast init`

    import pathlib
    from datetime import datetime, timedelta

    from feast.driver_test_data import create_driver_hourly_stats_df

    repo_path = pathlib.Path(__file__).parent.absolute() / "feature_repo"
    config_file = repo_path / "feature_store.yaml"

    if click.confirm("Configure Couchbase Online Store?", default=True):
        connection_string = click.prompt(
            "Couchbase Connection String", default="couchbase://127.0.0.1"
        )
        user = click.prompt("Couchbase Username", default="Administrator")
        password = click.prompt("Couchbase Password", hide_input=True)
        bucket_name = click.prompt("Couchbase Bucket Name", default="feast")
        kv_port = click.prompt("Couchbase KV Port", default=11210)

        replace_str_in_file(
            config_file, "COUCHBASE_CONNECTION_STRING", connection_string
        )
        replace_str_in_file(config_file, "COUCHBASE_USER", user)
        replace_str_in_file(config_file, "COUCHBASE_PASSWORD", password)
        replace_str_in_file(config_file, "COUCHBASE_BUCKET_NAME", bucket_name)
        replace_str_in_file(config_file, "COUCHBASE_KV_PORT", str(kv_port))

    if click.confirm(
        "Configure Couchbase Columnar Offline Store? (Note: requires Couchbase Capella Columnar)",
        default=True,
    ):
        end_date = datetime.now().replace(microsecond=0, second=0, minute=0)
        start_date = end_date - timedelta(days=15)

        driver_entities = [1001, 1002, 1003, 1004, 1005]
        driver_df = create_driver_hourly_stats_df(driver_entities, start_date, end_date)

        columnar_connection_string = click.prompt("Columnar Connection String")
        columnar_user = click.prompt("Columnar Username")
        columnar_password = click.prompt("Columnar Password", hide_input=True)
        columnar_timeout = click.prompt("Couchbase Columnar Timeout", default=120)

        if click.confirm(
            'Should I upload example data to Couchbase Capella Columnar (overwriting "Default.Default.feast_driver_hourly_stats" table)?',
            default=True,
        ):
            cred = Credential.from_username_and_password(
                columnar_user, columnar_password
            )
            timeout_opts = TimeoutOptions(dispatch_timeout=timedelta(seconds=120))
            cluster = Cluster.create_instance(
                columnar_connection_string,
                cred,
                ClusterOptions(timeout_options=timeout_opts),
            )

            table_name = "Default.Default.feast_driver_hourly_stats"
            try:
                cluster.execute_query(
                    f"DROP COLLECTION {table_name} IF EXISTS",
                    QueryOptions(timeout=timedelta(seconds=columnar_timeout)),
                )
            except InvalidCredentialError:
                print("Error: Invalid Cluster Credentials.")
                return

            offline_store = CouchbaseColumnarOfflineStoreConfig(
                type="couchbase.offline",
                connection_string=columnar_connection_string,
                user=columnar_user,
                password=columnar_password,
                timeout=columnar_timeout,
            )

            df_to_columnar(
                df=driver_df, table_name=table_name, offline_store=offline_store
            )

        replace_str_in_file(
            config_file,
            "COUCHBASE_COLUMNAR_CONNECTION_STRING",
            columnar_connection_string,
        )
        replace_str_in_file(config_file, "COUCHBASE_COLUMNAR_USER", columnar_user)
        replace_str_in_file(
            config_file, "COUCHBASE_COLUMNAR_PASSWORD", columnar_password
        )
        replace_str_in_file(
            config_file, "COUCHBASE_COLUMNAR_TIMEOUT", str(columnar_timeout)
        )


if __name__ == "__main__":
    bootstrap()



from typing import Union

from pyspark.sql import SparkSession

from feast.infra.common.materialization_job import MaterializationTask
from feast.infra.common.retrieval_task import HistoricalRetrievalTask
from feast.infra.compute_engines.feature_builder import FeatureBuilder
from feast.infra.compute_engines.spark.nodes import (
    SparkAggregationNode,
    SparkDedupNode,
    SparkFilterNode,
    SparkJoinNode,
    SparkReadNode,
    SparkTransformationNode,
    SparkWriteNode,
)
from feast.infra.registry.base_registry import BaseRegistry


class SparkFeatureBuilder(FeatureBuilder):
    def __init__(
        self,
        registry: BaseRegistry,
        spark_session: SparkSession,
        task: Union[MaterializationTask, HistoricalRetrievalTask],
    ):
        super().__init__(registry, task.feature_view, task)
        self.spark_session = spark_session

    def build_source_node(self, view):
        start_time = self.task.start_time
        end_time = self.task.end_time
        source = view.batch_source
        column_info = self.get_column_info(view)
        node = SparkReadNode(
            f"{view.name}:source",
            source,
            column_info,
            self.spark_session,
            start_time,
            end_time,
        )
        self.nodes.append(node)
        return node

    def build_aggregation_node(self, view, input_node):
        agg_specs = view.aggregations
        group_by_keys = view.entities
        timestamp_col = view.batch_source.timestamp_field
        node = SparkAggregationNode(
            f"{view.name}:agg",
            agg_specs,
            group_by_keys,
            timestamp_col,
            inputs=[input_node],
        )
        self.nodes.append(node)
        return node

    def build_join_node(self, view, input_nodes):
        column_info = self.get_column_info(view)
        node = SparkJoinNode(
            name=f"{view.name}:join",
            column_info=column_info,
            spark_session=self.spark_session,
            inputs=input_nodes,
            how="left",  # You can make this configurable later
        )
        self.nodes.append(node)
        return node

    def build_filter_node(self, view, input_node):
        filter_expr = getattr(view, "filter", None)
        ttl = getattr(view, "ttl", None)
        column_info = self.get_column_info(view)
        node = SparkFilterNode(
            f"{view.name}:filter",
            column_info,
            self.spark_session,
            ttl,
            filter_expr,
            inputs=[input_node],
        )
        self.nodes.append(node)
        return node

    def build_dedup_node(self, view, input_node):
        column_info = self.get_column_info(view)
        node = SparkDedupNode(
            f"{view.name}:dedup", column_info, self.spark_session, inputs=[input_node]
        )
        self.nodes.append(node)
        return node

    def build_transformation_node(self, view, input_nodes):
        udf_name = view.feature_transformation.name
        udf = view.feature_transformation.udf
        node = SparkTransformationNode(udf_name, udf, inputs=input_nodes)
        self.nodes.append(node)
        return node

    def build_output_nodes(self, view, input_node):
        node = SparkWriteNode(
            f"{view.name}:output", self.dag_root.view, inputs=[input_node]
        )
        self.nodes.append(node)
        return node

    def build_validation_node(self, view, input_node):
        pass



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file

Copyright 2020 The Feast Authors

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    https://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
import builtins
import collections.abc
import feast.core.Aggregation_pb2
import feast.core.DataSource_pb2
import feast.core.FeatureViewProjection_pb2
import feast.core.FeatureView_pb2
import feast.core.Feature_pb2
import feast.core.Transformation_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class OnDemandFeatureView(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SPEC_FIELD_NUMBER: builtins.int
    META_FIELD_NUMBER: builtins.int
    @property
    def spec(self) -> global___OnDemandFeatureViewSpec:
        """User-specified specifications of this feature view."""
    @property
    def meta(self) -> global___OnDemandFeatureViewMeta: ...
    def __init__(
        self,
        *,
        spec: global___OnDemandFeatureViewSpec | None = ...,
        meta: global___OnDemandFeatureViewMeta | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["meta", b"meta", "spec", b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["meta", b"meta", "spec", b"spec"]) -> None: ...

global___OnDemandFeatureView = OnDemandFeatureView

class OnDemandFeatureViewSpec(google.protobuf.message.Message):
    """Next available id: 9"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class SourcesEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___OnDemandSource: ...
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: global___OnDemandSource | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing_extensions.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    class TagsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        value: builtins.str
        def __init__(
            self,
            *,
            key: builtins.str = ...,
            value: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    PROJECT_FIELD_NUMBER: builtins.int
    FEATURES_FIELD_NUMBER: builtins.int
    SOURCES_FIELD_NUMBER: builtins.int
    USER_DEFINED_FUNCTION_FIELD_NUMBER: builtins.int
    FEATURE_TRANSFORMATION_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    TAGS_FIELD_NUMBER: builtins.int
    OWNER_FIELD_NUMBER: builtins.int
    MODE_FIELD_NUMBER: builtins.int
    WRITE_TO_ONLINE_STORE_FIELD_NUMBER: builtins.int
    ENTITIES_FIELD_NUMBER: builtins.int
    ENTITY_COLUMNS_FIELD_NUMBER: builtins.int
    SINGLETON_FIELD_NUMBER: builtins.int
    AGGREGATIONS_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of the feature view. Must be unique. Not updated."""
    project: builtins.str
    """Name of Feast project that this feature view belongs to."""
    @property
    def features(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Feature_pb2.FeatureSpecV2]:
        """List of features specifications for each feature defined with this feature view."""
    @property
    def sources(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___OnDemandSource]:
        """Map of sources for this feature view."""
    @property
    def user_defined_function(self) -> global___UserDefinedFunction: ...
    @property
    def feature_transformation(self) -> feast.core.Transformation_pb2.FeatureTransformationV2:
        """Oneof with {user_defined_function, on_demand_substrait_transformation}"""
    description: builtins.str
    """Description of the on demand feature view."""
    @property
    def tags(self) -> google.protobuf.internal.containers.ScalarMap[builtins.str, builtins.str]:
        """User defined metadata."""
    owner: builtins.str
    """Owner of the on demand feature view."""
    mode: builtins.str
    write_to_online_store: builtins.bool
    @property
    def entities(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """List of names of entities associated with this feature view."""
    @property
    def entity_columns(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Feature_pb2.FeatureSpecV2]:
        """List of specifications for each entity defined as part of this feature view."""
    singleton: builtins.bool
    @property
    def aggregations(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[feast.core.Aggregation_pb2.Aggregation]:
        """Aggregation definitions"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        project: builtins.str = ...,
        features: collections.abc.Iterable[feast.core.Feature_pb2.FeatureSpecV2] | None = ...,
        sources: collections.abc.Mapping[builtins.str, global___OnDemandSource] | None = ...,
        user_defined_function: global___UserDefinedFunction | None = ...,
        feature_transformation: feast.core.Transformation_pb2.FeatureTransformationV2 | None = ...,
        description: builtins.str = ...,
        tags: collections.abc.Mapping[builtins.str, builtins.str] | None = ...,
        owner: builtins.str = ...,
        mode: builtins.str = ...,
        write_to_online_store: builtins.bool = ...,
        entities: collections.abc.Iterable[builtins.str] | None = ...,
        entity_columns: collections.abc.Iterable[feast.core.Feature_pb2.FeatureSpecV2] | None = ...,
        singleton: builtins.bool = ...,
        aggregations: collections.abc.Iterable[feast.core.Aggregation_pb2.Aggregation] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["feature_transformation", b"feature_transformation", "user_defined_function", b"user_defined_function"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["aggregations", b"aggregations", "description", b"description", "entities", b"entities", "entity_columns", b"entity_columns", "feature_transformation", b"feature_transformation", "features", b"features", "mode", b"mode", "name", b"name", "owner", b"owner", "project", b"project", "singleton", b"singleton", "sources", b"sources", "tags", b"tags", "user_defined_function", b"user_defined_function", "write_to_online_store", b"write_to_online_store"]) -> None: ...

global___OnDemandFeatureViewSpec = OnDemandFeatureViewSpec

class OnDemandFeatureViewMeta(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATED_TIMESTAMP_FIELD_NUMBER: builtins.int
    LAST_UPDATED_TIMESTAMP_FIELD_NUMBER: builtins.int
    @property
    def created_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Time where this Feature View is created"""
    @property
    def last_updated_timestamp(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Time where this Feature View is last updated"""
    def __init__(
        self,
        *,
        created_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        last_updated_timestamp: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["created_timestamp", b"created_timestamp", "last_updated_timestamp", b"last_updated_timestamp"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["created_timestamp", b"created_timestamp", "last_updated_timestamp", b"last_updated_timestamp"]) -> None: ...

global___OnDemandFeatureViewMeta = OnDemandFeatureViewMeta

class OnDemandSource(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    FEATURE_VIEW_FIELD_NUMBER: builtins.int
    FEATURE_VIEW_PROJECTION_FIELD_NUMBER: builtins.int
    REQUEST_DATA_SOURCE_FIELD_NUMBER: builtins.int
    @property
    def feature_view(self) -> feast.core.FeatureView_pb2.FeatureView: ...
    @property
    def feature_view_projection(self) -> feast.core.FeatureViewProjection_pb2.FeatureViewProjection: ...
    @property
    def request_data_source(self) -> feast.core.DataSource_pb2.DataSource: ...
    def __init__(
        self,
        *,
        feature_view: feast.core.FeatureView_pb2.FeatureView | None = ...,
        feature_view_projection: feast.core.FeatureViewProjection_pb2.FeatureViewProjection | None = ...,
        request_data_source: feast.core.DataSource_pb2.DataSource | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["feature_view", b"feature_view", "feature_view_projection", b"feature_view_projection", "request_data_source", b"request_data_source", "source", b"source"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["feature_view", b"feature_view", "feature_view_projection", b"feature_view_projection", "request_data_source", b"request_data_source", "source", b"source"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["source", b"source"]) -> typing_extensions.Literal["feature_view", "feature_view_projection", "request_data_source"] | None: ...

global___OnDemandSource = OnDemandSource

class UserDefinedFunction(google.protobuf.message.Message):
    """Serialized representation of python function."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    BODY_FIELD_NUMBER: builtins.int
    BODY_TEXT_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The function name"""
    body: builtins.bytes
    """The python-syntax function body (serialized by dill)"""
    body_text: builtins.str
    """The string representation of the udf"""
    def __init__(
        self,
        *,
        name: builtins.str = ...,
        body: builtins.bytes = ...,
        body_text: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["body", b"body", "body_text", b"body_text", "name", b"name"]) -> None: ...

global___UserDefinedFunction = UserDefinedFunction

class OnDemandFeatureViewList(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ONDEMANDFEATUREVIEWS_FIELD_NUMBER: builtins.int
    @property
    def ondemandfeatureviews(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___OnDemandFeatureView]: ...
    def __init__(
        self,
        *,
        ondemandfeatureviews: collections.abc.Iterable[global___OnDemandFeatureView] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ondemandfeatureviews", b"ondemandfeatureviews"]) -> None: ...

global___OnDemandFeatureViewList = OnDemandFeatureViewList



from dataclasses import dataclass
from typing import List, Optional

import pyspark
from pyspark.sql import SparkSession

from feast import OnDemandFeatureView, RepoConfig
from feast.infra.common.materialization_job import (
    MaterializationJob,
    MaterializationJobStatus,
)
from feast.infra.compute_engines.dag.context import ExecutionContext
from feast.infra.compute_engines.dag.plan import ExecutionPlan
from feast.infra.offline_stores.contrib.spark_offline_store.spark import (
    SparkRetrievalJob,
)
from feast.infra.offline_stores.offline_store import RetrievalMetadata


class SparkDAGRetrievalJob(SparkRetrievalJob):
    def __init__(
        self,
        spark_session: SparkSession,
        plan: Optional[ExecutionPlan],
        context: ExecutionContext,
        full_feature_names: bool,
        config: RepoConfig,
        on_demand_feature_views: Optional[List[OnDemandFeatureView]] = None,
        metadata: Optional[RetrievalMetadata] = None,
        error: Optional[BaseException] = None,
    ):
        super().__init__(
            spark_session=spark_session,
            query="",
            full_feature_names=full_feature_names,
            config=config,
            on_demand_feature_views=on_demand_feature_views,
            metadata=metadata,
        )
        self._plan = plan
        self._context = context
        self._metadata = metadata
        self._spark_df = None
        self._error = error

    def error(self) -> Optional[BaseException]:
        return self._error

    def _ensure_executed(self):
        if self._spark_df is None:
            result = self._plan.execute(self._context)
            self._spark_df = result.data

    def to_spark_df(self) -> pyspark.sql.DataFrame:
        self._ensure_executed()
        assert self._spark_df is not None, "Execution plan did not produce a DataFrame"
        return self._spark_df

    def to_sql(self) -> str:
        assert self._plan is not None, "Execution plan is not set"
        return self._plan.to_sql(self._context)


@dataclass
class SparkMaterializationJob(MaterializationJob):
    def url(self) -> Optional[str]:
        pass

    def __init__(
        self,
        job_id: str,
        status: MaterializationJobStatus,
        error: Optional[BaseException] = None,
    ) -> None:
        super().__init__()
        self._job_id: str = job_id
        self._status: MaterializationJobStatus = status
        self._error: Optional[BaseException] = error

    def status(self) -> MaterializationJobStatus:
        return self._status

    def error(self) -> Optional[BaseException]:
        return self._error

    def should_be_retried(self) -> bool:
        return False

    def job_id(self) -> str:
        return self._job_id



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/core/DataFormat.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66\x65\x61st/core/DataFormat.proto\x12\nfeast.core\"\xb2\x01\n\nFileFormat\x12>\n\x0eparquet_format\x18\x01 \x01(\x0b\x32$.feast.core.FileFormat.ParquetFormatH\x00\x12:\n\x0c\x64\x65lta_format\x18\x02 \x01(\x0b\x32\".feast.core.FileFormat.DeltaFormatH\x00\x1a\x0f\n\rParquetFormat\x1a\r\n\x0b\x44\x65ltaFormatB\x08\n\x06\x66ormat\"\xb7\x02\n\x0cStreamFormat\x12:\n\x0b\x61vro_format\x18\x01 \x01(\x0b\x32#.feast.core.StreamFormat.AvroFormatH\x00\x12<\n\x0cproto_format\x18\x02 \x01(\x0b\x32$.feast.core.StreamFormat.ProtoFormatH\x00\x12:\n\x0bjson_format\x18\x03 \x01(\x0b\x32#.feast.core.StreamFormat.JsonFormatH\x00\x1a!\n\x0bProtoFormat\x12\x12\n\nclass_path\x18\x01 \x01(\t\x1a!\n\nAvroFormat\x12\x13\n\x0bschema_json\x18\x01 \x01(\t\x1a!\n\nJsonFormat\x12\x13\n\x0bschema_json\x18\x01 \x01(\tB\x08\n\x06\x66ormatBT\n\x10\x66\x65\x61st.proto.coreB\x0f\x44\x61taFormatProtoZ/github.com/feast-dev/feast/go/protos/feast/coreb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.core.DataFormat_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\020feast.proto.coreB\017DataFormatProtoZ/github.com/feast-dev/feast/go/protos/feast/core'
  _globals['_FILEFORMAT']._serialized_start=44
  _globals['_FILEFORMAT']._serialized_end=222
  _globals['_FILEFORMAT_PARQUETFORMAT']._serialized_start=182
  _globals['_FILEFORMAT_PARQUETFORMAT']._serialized_end=197
  _globals['_FILEFORMAT_DELTAFORMAT']._serialized_start=199
  _globals['_FILEFORMAT_DELTAFORMAT']._serialized_end=212
  _globals['_STREAMFORMAT']._serialized_start=225
  _globals['_STREAMFORMAT']._serialized_end=536
  _globals['_STREAMFORMAT_PROTOFORMAT']._serialized_start=423
  _globals['_STREAMFORMAT_PROTOFORMAT']._serialized_end=456
  _globals['_STREAMFORMAT_AVROFORMAT']._serialized_start=458
  _globals['_STREAMFORMAT_AVROFORMAT']._serialized_end=491
  _globals['_STREAMFORMAT_JSONFORMAT']._serialized_start=493
  _globals['_STREAMFORMAT_JSONFORMAT']._serialized_end=526
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: feast/registry/RegistryServer.proto
# Protobuf Python Version: 4.25.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from feast.protos.feast.core import Registry_pb2 as feast_dot_core_dot_Registry__pb2
from feast.protos.feast.core import Entity_pb2 as feast_dot_core_dot_Entity__pb2
from feast.protos.feast.core import DataSource_pb2 as feast_dot_core_dot_DataSource__pb2
from feast.protos.feast.core import FeatureView_pb2 as feast_dot_core_dot_FeatureView__pb2
from feast.protos.feast.core import StreamFeatureView_pb2 as feast_dot_core_dot_StreamFeatureView__pb2
from feast.protos.feast.core import OnDemandFeatureView_pb2 as feast_dot_core_dot_OnDemandFeatureView__pb2
from feast.protos.feast.core import FeatureService_pb2 as feast_dot_core_dot_FeatureService__pb2
from feast.protos.feast.core import SavedDataset_pb2 as feast_dot_core_dot_SavedDataset__pb2
from feast.protos.feast.core import ValidationProfile_pb2 as feast_dot_core_dot_ValidationProfile__pb2
from feast.protos.feast.core import InfraObject_pb2 as feast_dot_core_dot_InfraObject__pb2
from feast.protos.feast.core import Permission_pb2 as feast_dot_core_dot_Permission__pb2
from feast.protos.feast.core import Project_pb2 as feast_dot_core_dot_Project__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#feast/registry/RegistryServer.proto\x12\x0e\x66\x65\x61st.registry\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19\x66\x65\x61st/core/Registry.proto\x1a\x17\x66\x65\x61st/core/Entity.proto\x1a\x1b\x66\x65\x61st/core/DataSource.proto\x1a\x1c\x66\x65\x61st/core/FeatureView.proto\x1a\"feast/core/StreamFeatureView.proto\x1a$feast/core/OnDemandFeatureView.proto\x1a\x1f\x66\x65\x61st/core/FeatureService.proto\x1a\x1d\x66\x65\x61st/core/SavedDataset.proto\x1a\"feast/core/ValidationProfile.proto\x1a\x1c\x66\x65\x61st/core/InfraObject.proto\x1a\x1b\x66\x65\x61st/core/Permission.proto\x1a\x18\x66\x65\x61st/core/Project.proto\"/\n\x10PaginationParams\x12\x0c\n\x04page\x18\x01 \x01(\x05\x12\r\n\x05limit\x18\x02 \x01(\x05\"4\n\rSortingParams\x12\x0f\n\x07sort_by\x18\x01 \x01(\t\x12\x12\n\nsort_order\x18\x02 \x01(\t\"\x83\x01\n\x12PaginationMetadata\x12\x0c\n\x04page\x18\x01 \x01(\x05\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x13\n\x0btotal_count\x18\x03 \x01(\x05\x12\x13\n\x0btotal_pages\x18\x04 \x01(\x05\x12\x10\n\x08has_next\x18\x05 \x01(\x08\x12\x14\n\x0chas_previous\x18\x06 \x01(\x08\"!\n\x0eRefreshRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\"W\n\x12UpdateInfraRequest\x12 \n\x05infra\x18\x01 \x01(\x0b\x32\x11.feast.core.Infra\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"7\n\x0fGetInfraRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\"B\n\x1aListProjectMetadataRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\"T\n\x1bListProjectMetadataResponse\x12\x35\n\x10project_metadata\x18\x01 \x03(\x0b\x32\x1b.feast.core.ProjectMetadata\"\xcb\x01\n\x1b\x41pplyMaterializationRequest\x12-\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\x0b\x32\x17.feast.core.FeatureView\x12\x0f\n\x07project\x18\x02 \x01(\t\x12.\n\nstart_date\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12,\n\x08\x65nd_date\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06\x63ommit\x18\x05 \x01(\x08\"Y\n\x12\x41pplyEntityRequest\x12\"\n\x06\x65ntity\x18\x01 \x01(\x0b\x32\x12.feast.core.Entity\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"F\n\x10GetEntityRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\x8b\x02\n\x13ListEntitiesRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12;\n\x04tags\x18\x03 \x03(\x0b\x32-.feast.registry.ListEntitiesRequest.TagsEntry\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"t\n\x14ListEntitiesResponse\x12$\n\x08\x65ntities\x18\x01 \x03(\x0b\x32\x12.feast.core.Entity\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"D\n\x13\x44\x65leteEntityRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"f\n\x16\x41pplyDataSourceRequest\x12+\n\x0b\x64\x61ta_source\x18\x01 \x01(\x0b\x32\x16.feast.core.DataSource\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"J\n\x14GetDataSourceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\x91\x02\n\x16ListDataSourcesRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12>\n\x04tags\x18\x03 \x03(\x0b\x32\x30.feast.registry.ListDataSourcesRequest.TagsEntry\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x7f\n\x17ListDataSourcesResponse\x12,\n\x0c\x64\x61ta_sources\x18\x01 \x03(\x0b\x32\x16.feast.core.DataSource\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"H\n\x17\x44\x65leteDataSourceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"\x81\x02\n\x17\x41pplyFeatureViewRequest\x12/\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\x0b\x32\x17.feast.core.FeatureViewH\x00\x12\x41\n\x16on_demand_feature_view\x18\x02 \x01(\x0b\x32\x1f.feast.core.OnDemandFeatureViewH\x00\x12<\n\x13stream_feature_view\x18\x03 \x01(\x0b\x32\x1d.feast.core.StreamFeatureViewH\x00\x12\x0f\n\x07project\x18\x04 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x05 \x01(\x08\x42\x13\n\x11\x62\x61se_feature_view\"K\n\x15GetFeatureViewRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\x93\x02\n\x17ListFeatureViewsRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12?\n\x04tags\x18\x03 \x03(\x0b\x32\x31.feast.registry.ListFeatureViewsRequest.TagsEntry\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x82\x01\n\x18ListFeatureViewsResponse\x12.\n\rfeature_views\x18\x01 \x03(\x0b\x32\x17.feast.core.FeatureView\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"I\n\x18\x44\x65leteFeatureViewRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"\xd6\x01\n\x0e\x41nyFeatureView\x12/\n\x0c\x66\x65\x61ture_view\x18\x01 \x01(\x0b\x32\x17.feast.core.FeatureViewH\x00\x12\x41\n\x16on_demand_feature_view\x18\x02 \x01(\x0b\x32\x1f.feast.core.OnDemandFeatureViewH\x00\x12<\n\x13stream_feature_view\x18\x03 \x01(\x0b\x32\x1d.feast.core.StreamFeatureViewH\x00\x42\x12\n\x10\x61ny_feature_view\"N\n\x18GetAnyFeatureViewRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"U\n\x19GetAnyFeatureViewResponse\x12\x38\n\x10\x61ny_feature_view\x18\x01 \x01(\x0b\x32\x1e.feast.registry.AnyFeatureView\"\xe8\x02\n\x1aListAllFeatureViewsRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12\x42\n\x04tags\x18\x03 \x03(\x0b\x32\x34.feast.registry.ListAllFeatureViewsRequest.TagsEntry\x12\x0e\n\x06\x65ntity\x18\x04 \x01(\t\x12\x0f\n\x07\x66\x65\x61ture\x18\x05 \x01(\t\x12\x17\n\x0f\x66\x65\x61ture_service\x18\x06 \x01(\t\x12\x13\n\x0b\x64\x61ta_source\x18\x07 \x01(\t\x12\x34\n\npagination\x18\x08 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\t \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x8c\x01\n\x1bListAllFeatureViewsResponse\x12\x35\n\rfeature_views\x18\x01 \x03(\x0b\x32\x1e.feast.registry.AnyFeatureView\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"Q\n\x1bGetStreamFeatureViewRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\x9f\x02\n\x1dListStreamFeatureViewsRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12\x45\n\x04tags\x18\x03 \x03(\x0b\x32\x37.feast.registry.ListStreamFeatureViewsRequest.TagsEntry\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x95\x01\n\x1eListStreamFeatureViewsResponse\x12;\n\x14stream_feature_views\x18\x01 \x03(\x0b\x32\x1d.feast.core.StreamFeatureView\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"S\n\x1dGetOnDemandFeatureViewRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\xa3\x02\n\x1fListOnDemandFeatureViewsRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12G\n\x04tags\x18\x03 \x03(\x0b\x32\x39.feast.registry.ListOnDemandFeatureViewsRequest.TagsEntry\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x9c\x01\n ListOnDemandFeatureViewsResponse\x12@\n\x17on_demand_feature_views\x18\x01 \x03(\x0b\x32\x1f.feast.core.OnDemandFeatureView\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"r\n\x1a\x41pplyFeatureServiceRequest\x12\x33\n\x0f\x66\x65\x61ture_service\x18\x01 \x01(\x0b\x32\x1a.feast.core.FeatureService\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"N\n\x18GetFeatureServiceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\xaf\x02\n\x1aListFeatureServicesRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12\x42\n\x04tags\x18\x03 \x03(\x0b\x32\x34.feast.registry.ListFeatureServicesRequest.TagsEntry\x12\x14\n\x0c\x66\x65\x61ture_view\x18\x04 \x01(\t\x12\x34\n\npagination\x18\x05 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x06 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x8b\x01\n\x1bListFeatureServicesResponse\x12\x34\n\x10\x66\x65\x61ture_services\x18\x01 \x03(\x0b\x32\x1a.feast.core.FeatureService\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"L\n\x1b\x44\x65leteFeatureServiceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"l\n\x18\x41pplySavedDatasetRequest\x12/\n\rsaved_dataset\x18\x01 \x01(\x0b\x32\x18.feast.core.SavedDataset\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"L\n\x16GetSavedDatasetRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\x95\x02\n\x18ListSavedDatasetsRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12@\n\x04tags\x18\x03 \x03(\x0b\x32\x32.feast.registry.ListSavedDatasetsRequest.TagsEntry\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x85\x01\n\x19ListSavedDatasetsResponse\x12\x30\n\x0esaved_datasets\x18\x01 \x03(\x0b\x32\x18.feast.core.SavedDataset\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"J\n\x19\x44\x65leteSavedDatasetRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"\x81\x01\n\x1f\x41pplyValidationReferenceRequest\x12=\n\x14validation_reference\x18\x01 \x01(\x0b\x32\x1f.feast.core.ValidationReference\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"S\n\x1dGetValidationReferenceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\xa3\x02\n\x1fListValidationReferencesRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12G\n\x04tags\x18\x03 \x03(\x0b\x32\x39.feast.registry.ListValidationReferencesRequest.TagsEntry\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x9a\x01\n ListValidationReferencesResponse\x12>\n\x15validation_references\x18\x01 \x03(\x0b\x32\x1f.feast.core.ValidationReference\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"Q\n DeleteValidationReferenceRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"e\n\x16\x41pplyPermissionRequest\x12*\n\npermission\x18\x01 \x01(\x0b\x32\x16.feast.core.Permission\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"J\n\x14GetPermissionRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x03 \x01(\x08\"\x91\x02\n\x16ListPermissionsRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12>\n\x04tags\x18\x03 \x03(\x0b\x32\x30.feast.registry.ListPermissionsRequest.TagsEntry\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"~\n\x17ListPermissionsResponse\x12+\n\x0bpermissions\x18\x01 \x03(\x0b\x32\x16.feast.core.Permission\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"H\n\x17\x44\x65letePermissionRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07project\x18\x02 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x03 \x01(\x08\"K\n\x13\x41pplyProjectRequest\x12$\n\x07project\x18\x01 \x01(\x0b\x32\x13.feast.core.Project\x12\x0e\n\x06\x63ommit\x18\x02 \x01(\x08\"6\n\x11GetProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\"\xfa\x01\n\x13ListProjectsRequest\x12\x13\n\x0b\x61llow_cache\x18\x01 \x01(\x08\x12;\n\x04tags\x18\x02 \x03(\x0b\x32-.feast.registry.ListProjectsRequest.TagsEntry\x12\x34\n\npagination\x18\x03 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x04 \x01(\x0b\x32\x1d.feast.registry.SortingParams\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"u\n\x14ListProjectsResponse\x12%\n\x08projects\x18\x01 \x03(\x0b\x32\x13.feast.core.Project\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"4\n\x14\x44\x65leteProjectRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0e\n\x06\x63ommit\x18\x02 \x01(\x08\"-\n\x0f\x45ntityReference\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\"r\n\x0e\x45ntityRelation\x12/\n\x06source\x18\x01 \x01(\x0b\x32\x1f.feast.registry.EntityReference\x12/\n\x06target\x18\x02 \x01(\x0b\x32\x1f.feast.registry.EntityReference\"\xdf\x01\n\x19GetRegistryLineageRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x02 \x01(\x08\x12\x1a\n\x12\x66ilter_object_type\x18\x03 \x01(\t\x12\x1a\n\x12\x66ilter_object_name\x18\x04 \x01(\t\x12\x34\n\npagination\x18\x05 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x06 \x01(\x0b\x32\x1d.feast.registry.SortingParams\"\xa8\x02\n\x1aGetRegistryLineageResponse\x12\x35\n\rrelationships\x18\x01 \x03(\x0b\x32\x1e.feast.registry.EntityRelation\x12>\n\x16indirect_relationships\x18\x02 \x03(\x0b\x32\x1e.feast.registry.EntityRelation\x12\x44\n\x18relationships_pagination\x18\x03 \x01(\x0b\x32\".feast.registry.PaginationMetadata\x12M\n!indirect_relationships_pagination\x18\x04 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"\xef\x01\n\x1dGetObjectRelationshipsRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x13\n\x0bobject_type\x18\x02 \x01(\t\x12\x13\n\x0bobject_name\x18\x03 \x01(\t\x12\x18\n\x10include_indirect\x18\x04 \x01(\x08\x12\x13\n\x0b\x61llow_cache\x18\x05 \x01(\x08\x12\x34\n\npagination\x18\x06 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x07 \x01(\x0b\x32\x1d.feast.registry.SortingParams\"\x8f\x01\n\x1eGetObjectRelationshipsResponse\x12\x35\n\rrelationships\x18\x01 \x03(\x0b\x32\x1e.feast.registry.EntityRelation\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"\xb0\x02\n\x07\x46\x65\x61ture\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x14\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x04 \x01(\t\x12\r\n\x05owner\x18\x05 \x01(\t\x12\x35\n\x11\x63reated_timestamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12:\n\x16last_updated_timestamp\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x04tags\x18\x08 \x03(\x0b\x32!.feast.registry.Feature.TagsEntry\x1a+\n\tTagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xc5\x01\n\x13ListFeaturesRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x14\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x06 \x01(\x08\x12\x34\n\npagination\x18\x04 \x01(\x0b\x32 .feast.registry.PaginationParams\x12.\n\x07sorting\x18\x05 \x01(\x0b\x32\x1d.feast.registry.SortingParams\"y\n\x14ListFeaturesResponse\x12)\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x17.feast.registry.Feature\x12\x36\n\npagination\x18\x02 \x01(\x0b\x32\".feast.registry.PaginationMetadata\"]\n\x11GetFeatureRequest\x12\x0f\n\x07project\x18\x01 \x01(\t\x12\x14\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\t\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\x13\n\x0b\x61llow_cache\x18\x04 \x01(\x08\x32\xde#\n\x0eRegistryServer\x12K\n\x0b\x41pplyEntity\x12\".feast.registry.ApplyEntityRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x43\n\tGetEntity\x12 .feast.registry.GetEntityRequest\x1a\x12.feast.core.Entity\"\x00\x12[\n\x0cListEntities\x12#.feast.registry.ListEntitiesRequest\x1a$.feast.registry.ListEntitiesResponse\"\x00\x12M\n\x0c\x44\x65leteEntity\x12#.feast.registry.DeleteEntityRequest\x1a\x16.google.protobuf.Empty\"\x00\x12S\n\x0f\x41pplyDataSource\x12&.feast.registry.ApplyDataSourceRequest\x1a\x16.google.protobuf.Empty\"\x00\x12O\n\rGetDataSource\x12$.feast.registry.GetDataSourceRequest\x1a\x16.feast.core.DataSource\"\x00\x12\x64\n\x0fListDataSources\x12&.feast.registry.ListDataSourcesRequest\x1a\'.feast.registry.ListDataSourcesResponse\"\x00\x12U\n\x10\x44\x65leteDataSource\x12\'.feast.registry.DeleteDataSourceRequest\x1a\x16.google.protobuf.Empty\"\x00\x12U\n\x10\x41pplyFeatureView\x12\'.feast.registry.ApplyFeatureViewRequest\x1a\x16.google.protobuf.Empty\"\x00\x12W\n\x11\x44\x65leteFeatureView\x12(.feast.registry.DeleteFeatureViewRequest\x1a\x16.google.protobuf.Empty\"\x00\x12j\n\x11GetAnyFeatureView\x12(.feast.registry.GetAnyFeatureViewRequest\x1a).feast.registry.GetAnyFeatureViewResponse\"\x00\x12p\n\x13ListAllFeatureViews\x12*.feast.registry.ListAllFeatureViewsRequest\x1a+.feast.registry.ListAllFeatureViewsResponse\"\x00\x12R\n\x0eGetFeatureView\x12%.feast.registry.GetFeatureViewRequest\x1a\x17.feast.core.FeatureView\"\x00\x12g\n\x10ListFeatureViews\x12\'.feast.registry.ListFeatureViewsRequest\x1a(.feast.registry.ListFeatureViewsResponse\"\x00\x12\x64\n\x14GetStreamFeatureView\x12+.feast.registry.GetStreamFeatureViewRequest\x1a\x1d.feast.core.StreamFeatureView\"\x00\x12y\n\x16ListStreamFeatureViews\x12-.feast.registry.ListStreamFeatureViewsRequest\x1a..feast.registry.ListStreamFeatureViewsResponse\"\x00\x12j\n\x16GetOnDemandFeatureView\x12-.feast.registry.GetOnDemandFeatureViewRequest\x1a\x1f.feast.core.OnDemandFeatureView\"\x00\x12\x7f\n\x18ListOnDemandFeatureViews\x12/.feast.registry.ListOnDemandFeatureViewsRequest\x1a\x30.feast.registry.ListOnDemandFeatureViewsResponse\"\x00\x12[\n\x13\x41pplyFeatureService\x12*.feast.registry.ApplyFeatureServiceRequest\x1a\x16.google.protobuf.Empty\"\x00\x12[\n\x11GetFeatureService\x12(.feast.registry.GetFeatureServiceRequest\x1a\x1a.feast.core.FeatureService\"\x00\x12p\n\x13ListFeatureServices\x12*.feast.registry.ListFeatureServicesRequest\x1a+.feast.registry.ListFeatureServicesResponse\"\x00\x12]\n\x14\x44\x65leteFeatureService\x12+.feast.registry.DeleteFeatureServiceRequest\x1a\x16.google.protobuf.Empty\"\x00\x12W\n\x11\x41pplySavedDataset\x12(.feast.registry.ApplySavedDatasetRequest\x1a\x16.google.protobuf.Empty\"\x00\x12U\n\x0fGetSavedDataset\x12&.feast.registry.GetSavedDatasetRequest\x1a\x18.feast.core.SavedDataset\"\x00\x12j\n\x11ListSavedDatasets\x12(.feast.registry.ListSavedDatasetsRequest\x1a).feast.registry.ListSavedDatasetsResponse\"\x00\x12Y\n\x12\x44\x65leteSavedDataset\x12).feast.registry.DeleteSavedDatasetRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x65\n\x18\x41pplyValidationReference\x12/.feast.registry.ApplyValidationReferenceRequest\x1a\x16.google.protobuf.Empty\"\x00\x12j\n\x16GetValidationReference\x12-.feast.registry.GetValidationReferenceRequest\x1a\x1f.feast.core.ValidationReference\"\x00\x12\x7f\n\x18ListValidationReferences\x12/.feast.registry.ListValidationReferencesRequest\x1a\x30.feast.registry.ListValidationReferencesResponse\"\x00\x12g\n\x19\x44\x65leteValidationReference\x12\x30.feast.registry.DeleteValidationReferenceRequest\x1a\x16.google.protobuf.Empty\"\x00\x12S\n\x0f\x41pplyPermission\x12&.feast.registry.ApplyPermissionRequest\x1a\x16.google.protobuf.Empty\"\x00\x12O\n\rGetPermission\x12$.feast.registry.GetPermissionRequest\x1a\x16.feast.core.Permission\"\x00\x12\x64\n\x0fListPermissions\x12&.feast.registry.ListPermissionsRequest\x1a\'.feast.registry.ListPermissionsResponse\"\x00\x12U\n\x10\x44\x65letePermission\x12\'.feast.registry.DeletePermissionRequest\x1a\x16.google.protobuf.Empty\"\x00\x12M\n\x0c\x41pplyProject\x12#.feast.registry.ApplyProjectRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x46\n\nGetProject\x12!.feast.registry.GetProjectRequest\x1a\x13.feast.core.Project\"\x00\x12[\n\x0cListProjects\x12#.feast.registry.ListProjectsRequest\x1a$.feast.registry.ListProjectsResponse\"\x00\x12O\n\rDeleteProject\x12$.feast.registry.DeleteProjectRequest\x1a\x16.google.protobuf.Empty\"\x00\x12]\n\x14\x41pplyMaterialization\x12+.feast.registry.ApplyMaterializationRequest\x1a\x16.google.protobuf.Empty\"\x00\x12p\n\x13ListProjectMetadata\x12*.feast.registry.ListProjectMetadataRequest\x1a+.feast.registry.ListProjectMetadataResponse\"\x00\x12K\n\x0bUpdateInfra\x12\".feast.registry.UpdateInfraRequest\x1a\x16.google.protobuf.Empty\"\x00\x12@\n\x08GetInfra\x12\x1f.feast.registry.GetInfraRequest\x1a\x11.feast.core.Infra\"\x00\x12:\n\x06\x43ommit\x12\x16.google.protobuf.Empty\x1a\x16.google.protobuf.Empty\"\x00\x12\x43\n\x07Refresh\x12\x1e.feast.registry.RefreshRequest\x1a\x16.google.protobuf.Empty\"\x00\x12\x37\n\x05Proto\x12\x16.google.protobuf.Empty\x1a\x14.feast.core.Registry\"\x00\x12m\n\x12GetRegistryLineage\x12).feast.registry.GetRegistryLineageRequest\x1a*.feast.registry.GetRegistryLineageResponse\"\x00\x12y\n\x16GetObjectRelationships\x12-.feast.registry.GetObjectRelationshipsRequest\x1a..feast.registry.GetObjectRelationshipsResponse\"\x00\x12[\n\x0cListFeatures\x12#.feast.registry.ListFeaturesRequest\x1a$.feast.registry.ListFeaturesResponse\"\x00\x12J\n\nGetFeature\x12!.feast.registry.GetFeatureRequest\x1a\x17.feast.registry.Feature\"\x00\x42\x35Z3github.com/feast-dev/feast/go/protos/feast/registryb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'feast.registry.RegistryServer_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z3github.com/feast-dev/feast/go/protos/feast/registry'
  _globals['_LISTENTITIESREQUEST_TAGSENTRY']._options = None
  _globals['_LISTENTITIESREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTDATASOURCESREQUEST_TAGSENTRY']._options = None
  _globals['_LISTDATASOURCESREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTFEATUREVIEWSREQUEST_TAGSENTRY']._options = None
  _globals['_LISTFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTALLFEATUREVIEWSREQUEST_TAGSENTRY']._options = None
  _globals['_LISTALLFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTSTREAMFEATUREVIEWSREQUEST_TAGSENTRY']._options = None
  _globals['_LISTSTREAMFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTONDEMANDFEATUREVIEWSREQUEST_TAGSENTRY']._options = None
  _globals['_LISTONDEMANDFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTFEATURESERVICESREQUEST_TAGSENTRY']._options = None
  _globals['_LISTFEATURESERVICESREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTSAVEDDATASETSREQUEST_TAGSENTRY']._options = None
  _globals['_LISTSAVEDDATASETSREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTVALIDATIONREFERENCESREQUEST_TAGSENTRY']._options = None
  _globals['_LISTVALIDATIONREFERENCESREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTPERMISSIONSREQUEST_TAGSENTRY']._options = None
  _globals['_LISTPERMISSIONSREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_LISTPROJECTSREQUEST_TAGSENTRY']._options = None
  _globals['_LISTPROJECTSREQUEST_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_FEATURE_TAGSENTRY']._options = None
  _globals['_FEATURE_TAGSENTRY']._serialized_options = b'8\001'
  _globals['_PAGINATIONPARAMS']._serialized_start=487
  _globals['_PAGINATIONPARAMS']._serialized_end=534
  _globals['_SORTINGPARAMS']._serialized_start=536
  _globals['_SORTINGPARAMS']._serialized_end=588
  _globals['_PAGINATIONMETADATA']._serialized_start=591
  _globals['_PAGINATIONMETADATA']._serialized_end=722
  _globals['_REFRESHREQUEST']._serialized_start=724
  _globals['_REFRESHREQUEST']._serialized_end=757
  _globals['_UPDATEINFRAREQUEST']._serialized_start=759
  _globals['_UPDATEINFRAREQUEST']._serialized_end=846
  _globals['_GETINFRAREQUEST']._serialized_start=848
  _globals['_GETINFRAREQUEST']._serialized_end=903
  _globals['_LISTPROJECTMETADATAREQUEST']._serialized_start=905
  _globals['_LISTPROJECTMETADATAREQUEST']._serialized_end=971
  _globals['_LISTPROJECTMETADATARESPONSE']._serialized_start=973
  _globals['_LISTPROJECTMETADATARESPONSE']._serialized_end=1057
  _globals['_APPLYMATERIALIZATIONREQUEST']._serialized_start=1060
  _globals['_APPLYMATERIALIZATIONREQUEST']._serialized_end=1263
  _globals['_APPLYENTITYREQUEST']._serialized_start=1265
  _globals['_APPLYENTITYREQUEST']._serialized_end=1354
  _globals['_GETENTITYREQUEST']._serialized_start=1356
  _globals['_GETENTITYREQUEST']._serialized_end=1426
  _globals['_LISTENTITIESREQUEST']._serialized_start=1429
  _globals['_LISTENTITIESREQUEST']._serialized_end=1696
  _globals['_LISTENTITIESREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTENTITIESREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTENTITIESRESPONSE']._serialized_start=1698
  _globals['_LISTENTITIESRESPONSE']._serialized_end=1814
  _globals['_DELETEENTITYREQUEST']._serialized_start=1816
  _globals['_DELETEENTITYREQUEST']._serialized_end=1884
  _globals['_APPLYDATASOURCEREQUEST']._serialized_start=1886
  _globals['_APPLYDATASOURCEREQUEST']._serialized_end=1988
  _globals['_GETDATASOURCEREQUEST']._serialized_start=1990
  _globals['_GETDATASOURCEREQUEST']._serialized_end=2064
  _globals['_LISTDATASOURCESREQUEST']._serialized_start=2067
  _globals['_LISTDATASOURCESREQUEST']._serialized_end=2340
  _globals['_LISTDATASOURCESREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTDATASOURCESREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTDATASOURCESRESPONSE']._serialized_start=2342
  _globals['_LISTDATASOURCESRESPONSE']._serialized_end=2469
  _globals['_DELETEDATASOURCEREQUEST']._serialized_start=2471
  _globals['_DELETEDATASOURCEREQUEST']._serialized_end=2543
  _globals['_APPLYFEATUREVIEWREQUEST']._serialized_start=2546
  _globals['_APPLYFEATUREVIEWREQUEST']._serialized_end=2803
  _globals['_GETFEATUREVIEWREQUEST']._serialized_start=2805
  _globals['_GETFEATUREVIEWREQUEST']._serialized_end=2880
  _globals['_LISTFEATUREVIEWSREQUEST']._serialized_start=2883
  _globals['_LISTFEATUREVIEWSREQUEST']._serialized_end=3158
  _globals['_LISTFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTFEATUREVIEWSRESPONSE']._serialized_start=3161
  _globals['_LISTFEATUREVIEWSRESPONSE']._serialized_end=3291
  _globals['_DELETEFEATUREVIEWREQUEST']._serialized_start=3293
  _globals['_DELETEFEATUREVIEWREQUEST']._serialized_end=3366
  _globals['_ANYFEATUREVIEW']._serialized_start=3369
  _globals['_ANYFEATUREVIEW']._serialized_end=3583
  _globals['_GETANYFEATUREVIEWREQUEST']._serialized_start=3585
  _globals['_GETANYFEATUREVIEWREQUEST']._serialized_end=3663
  _globals['_GETANYFEATUREVIEWRESPONSE']._serialized_start=3665
  _globals['_GETANYFEATUREVIEWRESPONSE']._serialized_end=3750
  _globals['_LISTALLFEATUREVIEWSREQUEST']._serialized_start=3753
  _globals['_LISTALLFEATUREVIEWSREQUEST']._serialized_end=4113
  _globals['_LISTALLFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTALLFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTALLFEATUREVIEWSRESPONSE']._serialized_start=4116
  _globals['_LISTALLFEATUREVIEWSRESPONSE']._serialized_end=4256
  _globals['_GETSTREAMFEATUREVIEWREQUEST']._serialized_start=4258
  _globals['_GETSTREAMFEATUREVIEWREQUEST']._serialized_end=4339
  _globals['_LISTSTREAMFEATUREVIEWSREQUEST']._serialized_start=4342
  _globals['_LISTSTREAMFEATUREVIEWSREQUEST']._serialized_end=4629
  _globals['_LISTSTREAMFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTSTREAMFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTSTREAMFEATUREVIEWSRESPONSE']._serialized_start=4632
  _globals['_LISTSTREAMFEATUREVIEWSRESPONSE']._serialized_end=4781
  _globals['_GETONDEMANDFEATUREVIEWREQUEST']._serialized_start=4783
  _globals['_GETONDEMANDFEATUREVIEWREQUEST']._serialized_end=4866
  _globals['_LISTONDEMANDFEATUREVIEWSREQUEST']._serialized_start=4869
  _globals['_LISTONDEMANDFEATUREVIEWSREQUEST']._serialized_end=5160
  _globals['_LISTONDEMANDFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTONDEMANDFEATUREVIEWSREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTONDEMANDFEATUREVIEWSRESPONSE']._serialized_start=5163
  _globals['_LISTONDEMANDFEATUREVIEWSRESPONSE']._serialized_end=5319
  _globals['_APPLYFEATURESERVICEREQUEST']._serialized_start=5321
  _globals['_APPLYFEATURESERVICEREQUEST']._serialized_end=5435
  _globals['_GETFEATURESERVICEREQUEST']._serialized_start=5437
  _globals['_GETFEATURESERVICEREQUEST']._serialized_end=5515
  _globals['_LISTFEATURESERVICESREQUEST']._serialized_start=5518
  _globals['_LISTFEATURESERVICESREQUEST']._serialized_end=5821
  _globals['_LISTFEATURESERVICESREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTFEATURESERVICESREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTFEATURESERVICESRESPONSE']._serialized_start=5824
  _globals['_LISTFEATURESERVICESRESPONSE']._serialized_end=5963
  _globals['_DELETEFEATURESERVICEREQUEST']._serialized_start=5965
  _globals['_DELETEFEATURESERVICEREQUEST']._serialized_end=6041
  _globals['_APPLYSAVEDDATASETREQUEST']._serialized_start=6043
  _globals['_APPLYSAVEDDATASETREQUEST']._serialized_end=6151
  _globals['_GETSAVEDDATASETREQUEST']._serialized_start=6153
  _globals['_GETSAVEDDATASETREQUEST']._serialized_end=6229
  _globals['_LISTSAVEDDATASETSREQUEST']._serialized_start=6232
  _globals['_LISTSAVEDDATASETSREQUEST']._serialized_end=6509
  _globals['_LISTSAVEDDATASETSREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTSAVEDDATASETSREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTSAVEDDATASETSRESPONSE']._serialized_start=6512
  _globals['_LISTSAVEDDATASETSRESPONSE']._serialized_end=6645
  _globals['_DELETESAVEDDATASETREQUEST']._serialized_start=6647
  _globals['_DELETESAVEDDATASETREQUEST']._serialized_end=6721
  _globals['_APPLYVALIDATIONREFERENCEREQUEST']._serialized_start=6724
  _globals['_APPLYVALIDATIONREFERENCEREQUEST']._serialized_end=6853
  _globals['_GETVALIDATIONREFERENCEREQUEST']._serialized_start=6855
  _globals['_GETVALIDATIONREFERENCEREQUEST']._serialized_end=6938
  _globals['_LISTVALIDATIONREFERENCESREQUEST']._serialized_start=6941
  _globals['_LISTVALIDATIONREFERENCESREQUEST']._serialized_end=7232
  _globals['_LISTVALIDATIONREFERENCESREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTVALIDATIONREFERENCESREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTVALIDATIONREFERENCESRESPONSE']._serialized_start=7235
  _globals['_LISTVALIDATIONREFERENCESRESPONSE']._serialized_end=7389
  _globals['_DELETEVALIDATIONREFERENCEREQUEST']._serialized_start=7391
  _globals['_DELETEVALIDATIONREFERENCEREQUEST']._serialized_end=7472
  _globals['_APPLYPERMISSIONREQUEST']._serialized_start=7474
  _globals['_APPLYPERMISSIONREQUEST']._serialized_end=7575
  _globals['_GETPERMISSIONREQUEST']._serialized_start=7577
  _globals['_GETPERMISSIONREQUEST']._serialized_end=7651
  _globals['_LISTPERMISSIONSREQUEST']._serialized_start=7654
  _globals['_LISTPERMISSIONSREQUEST']._serialized_end=7927
  _globals['_LISTPERMISSIONSREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTPERMISSIONSREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTPERMISSIONSRESPONSE']._serialized_start=7929
  _globals['_LISTPERMISSIONSRESPONSE']._serialized_end=8055
  _globals['_DELETEPERMISSIONREQUEST']._serialized_start=8057
  _globals['_DELETEPERMISSIONREQUEST']._serialized_end=8129
  _globals['_APPLYPROJECTREQUEST']._serialized_start=8131
  _globals['_APPLYPROJECTREQUEST']._serialized_end=8206
  _globals['_GETPROJECTREQUEST']._serialized_start=8208
  _globals['_GETPROJECTREQUEST']._serialized_end=8262
  _globals['_LISTPROJECTSREQUEST']._serialized_start=8265
  _globals['_LISTPROJECTSREQUEST']._serialized_end=8515
  _globals['_LISTPROJECTSREQUEST_TAGSENTRY']._serialized_start=1653
  _globals['_LISTPROJECTSREQUEST_TAGSENTRY']._serialized_end=1696
  _globals['_LISTPROJECTSRESPONSE']._serialized_start=8517
  _globals['_LISTPROJECTSRESPONSE']._serialized_end=8634
  _globals['_DELETEPROJECTREQUEST']._serialized_start=8636
  _globals['_DELETEPROJECTREQUEST']._serialized_end=8688
  _globals['_ENTITYREFERENCE']._serialized_start=8690
  _globals['_ENTITYREFERENCE']._serialized_end=8735
  _globals['_ENTITYRELATION']._serialized_start=8737
  _globals['_ENTITYRELATION']._serialized_end=8851
  _globals['_GETREGISTRYLINEAGEREQUEST']._serialized_start=8854
  _globals['_GETREGISTRYLINEAGEREQUEST']._serialized_end=9077
  _globals['_GETREGISTRYLINEAGERESPONSE']._serialized_start=9080
  _globals['_GETREGISTRYLINEAGERESPONSE']._serialized_end=9376
  _globals['_GETOBJECTRELATIONSHIPSREQUEST']._serialized_start=9379
  _globals['_GETOBJECTRELATIONSHIPSREQUEST']._serialized_end=9618
  _globals['_GETOBJECTRELATIONSHIPSRESPONSE']._serialized_start=9621
  _globals['_GETOBJECTRELATIONSHIPSRESPONSE']._serialized_end=9764
  _globals['_FEATURE']._serialized_start=9767
  _globals['_FEATURE']._serialized_end=10071
  _globals['_FEATURE_TAGSENTRY']._serialized_start=1653
  _globals['_FEATURE_TAGSENTRY']._serialized_end=1696
  _globals['_LISTFEATURESREQUEST']._serialized_start=10074
  _globals['_LISTFEATURESREQUEST']._serialized_end=10271
  _globals['_LISTFEATURESRESPONSE']._serialized_start=10273
  _globals['_LISTFEATURESRESPONSE']._serialized_end=10394
  _globals['_GETFEATUREREQUEST']._serialized_start=10396
  _globals['_GETFEATUREREQUEST']._serialized_end=10489
  _globals['_REGISTRYSERVER']._serialized_start=10492
  _globals['_REGISTRYSERVER']._serialized_end=15066
# @@protoc_insertion_point(module_scope)
